/*****************************************************************************/
/*   Copyright 1996 National Instruments Corporation.  All Rights Reserved.  */
/*****************************************************************************/

#include <formatio.h>
#include <utility.h>
#include <visa.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "rssmp.h"

#define rssmp_REVISION     "Rev 2.4, 01/2002, CVI 5.0.1" /*  Instrument driver revision */
#define BUFFER_SIZE        4096L                    /* File I/O buffer size */

/*= Macros ==================================================================*/
#define CHECKERR_CU(fCal) if (rssmp_status = (fCal), rssmp_status < VI_SUCCESS) \
	return rssmp_initCleanUp (rmSession, instrSession, rssmp_status); else

/*===========================================================================*/
/*= Rohde & Schwarz SMP02, SMP03, & SMP04 Signal Generators =================*/
/* LabWindows/CVI 3.1 Instrument Driver                                      */
/* Original Release: October 1996                                            */
/* By: Michal Harhaj                                                         */
/*     Ph.: +420 69 6996146, Fax +420 69 6996147							 */
/*     Email: Michal.Harhaj@elcom.cz										 */
/*                                                                           */
/* For LabWindows Technical Support in the United States call:               */
/*     National Instruments, Austin, Texas                                   */
/*     Ph (800) 433-3488    Fax (512) 794-5678                               */
/*                                                                           */
/* Modification History:                                                     */
/*																			 */
/* January 2002:   Rev 2.4, 01/2002										     */
/*               - Changed Buffer-size to 4096L								 */
/*                   2048L is too small for user-correction with UNITS(MHz)	 */
/*				 - rssmp_confUserCorr       								 */
/*                  changed frequency correction values to MHz               */
/*               By: Thomas.Reuter, Philips,                                 */
/*                   Werner Klotzsche, Rohde & Schwarz                       */
/* March 2000:   Rev 2.3, 03/2000										     */
/*               - Changed data type for input string parameters             */
/*                 from ViChar _VI_FAR xxx[] to ViString xxx.                */
/*                 It's important for HP-VEE                                 */
/*               By: Zdenek Rykala, ELCOM, a.s., Czech Republic				 */
/*                                                                           */
/* February 2000:        Rev 2.2, 02/2000, CVI 5.0.1                         */
/*                       1. Added ASRL interface support                     */
/*                          - rssmp_init                                     */
/*                          - rssmp_operationComplete                        */
/*                       2. functionrssmp_confLFSweep - corrected formatting */
/*                          string within viPrintf function (missing step    */
/*                            parameter)                                     */
/*                       3. rssmp_confCreateLst - internaly redesigned to be */
/*                         working with firmware version "3.7". Replaced     */
/*                         commands "FORM:DATA PACK" and "BORD NORM".        */
/*                       4. Removed range checks dependand on model type or  */
/*                         included options. Now driver is using             */
/*                         min/max possible ranges. If parameter is related  */
/*                         to the offset (especially if offset is <> 0.0),   */
/*                         then range checking is fully ommited. In case of  */
/*                         OVER or UNDER range values instrument is returning*/
/*                         an error code. This change is related only to the */
/*                         some of frequency and level parameters,           */
/*                         if required.                                      */
/*                      5. Added Compatibility mode to SMR. Driver is        */
/*                         providing certain level of interchangeability     */
/*                         from SMP --> SMR instruments. Driver recognizes   */
/*                         instrument model type and makes functions working */
/*                         correctly in case of using SMR. For full support  */
/*                         of SMR has to be used new SMR driver.             */
/*                         Compatibility has been tested with SMR firmware   */
/*                         version 1.06 and solves known firmware bugs.      */
/*                      Modified by: Martin Koutny, ELCOM, a.s., CZ          */
/*                                                                           */
/* November 1999:       Rev 2.1, 11/1999, CVI 5.0.1                          */
/*                      1. Removed warnings                                  */
/*                      2. Changed to the latest VXIpnp standard             */
/*                         Following functions was renamed                   */
/*                         - rssmp_errorQuery to rssmp_error_query           */
/*                         - rssmp_errorMessage to rssmp_error_message       */
/*                         - rssmp_revisionQuery to rssmp_revision_query     */
/*                         - rssmp_selfTest to rssmp_self_test               */
/*                      3. Fixed function rssmp_self_test function           */
/*                      Modified by: Zdenek Rykala, ELCOM, a.s., CZ          */
/*                                                                           */
/* March 1999:          Rev 2.0, 03/1999, CVI 5.0.1                          */
/*                      1. Removed bug in array oneOrTwo                     */
/*                      2. Function Pulse Modulation Delay renamed to Pulse  */
/*                         Modulator Delay                                   */
/*                      3. Function Pulse Modulation Width renamed to Pulse  */
/*                         Modulator Width                                   */
/*                      4. Function Pulse Modulation Period renamed to Pulse */
/*                         Modulator Period                                  */
/*                      5. Removed bug in function rssmp_qLFfreq             */
/*                      Modified by: Michal Harhaj, ELCOM, a.s., CZ          */
/*                                                                           */
/* January 1999:         Added New functions (fast configuration functions   */
/*                       with a single parameter). Functions added at the    */
/*                       request of Rohde & Schwarz.                         */
/*                       RF Frequency              rssmp_qRFfrequency        */
/*                       RF Level                  rssmp_qRFlevel            */
/*                       RF Level Offset           rssmp_qRFlevOffset        */
/*                       RF Start Stop Frequency   rssmp_qRFstartStopFreq    */
/*                       RF Center Span Frequency  rssmp_qRFcenterSpanFreq   */
/*                       RF Sweep Step             rssmp_qRFsweepStep        */
/*                       LF Frequency              rssmp_qLFfreq             */
/*                       LF Voltage                rssmp_qLFvolt             */
/*                       AM Modulation Depth       rssmp_qAMmodDep           */
/*                       FM Deviation              rssmp_qFMdev              */
/*                       Phase Mod Deviation       rssmp_qPHMdev             */
/*                       Pulse Modulation Delay    rssmp_qPulsModDelay       */
/*                       Pulse Modulation Period   rssmp_qPulsModPeriod      */
/*                       Pulse Modulation Width    rssmp_qPulsModWidth       */
/*                       Modified by: Jan Sima, TU Ostrava, Czech Republic   */
/*                                                                           */
/* June 1997:            Modified the #defines for the constants for         */
/*                       status codes for parameters 9 to 15                 */
/*                       Modified by: TKS, National Instruments              */
/*===========================================================================*/
 
/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================*/
/*****************************************************************************/
static ViString attrArr[] = {"AUTO","FIX"};      
static ViString unitArr[] = {"DBM","DBW","DBMW","DBUW","DBV","DBMV","DBUV","V",
							 "" /* No units */};
static ViString ALCSourceArr[] = {"INT","DIOD","PMET"};
static ViString stateArr[] = {"OFF","ON"};
static ViString pMeterArr[] = {"RS_NRVS","HP436A","NRVD","NRVS"};
static ViString onceArr[] = {"ONCE","ON","OFF"};
static ViString AMSourceArr [] = {"INT1","INT2","EXT"};
static ViString FMPMSourceArr [] = {"INT","EXT1","EXT2"};
static ViString couplingArr [] = {"AC","DC"};
static ViString extImpedanceArr[] = {"100KOHM","600OHM"};
static ViString modulModeArr[] = {"UNL","LOCK","PREC"};
static ViString pulMExtImpArr [] = {"10E3","50"};
static ViString polarityArr[] = {"NORM","INV"};
static ViString pulsePerFreqArr[] = {":SOUR:PULS:PER",":SOUR:PULM:INT:FREQ"};
static ViString DMTypeArr[] = {"FSK","ASK"};
static ViString sweBlankTmArr[] = {"NORM","LONG"};
static ViString spacingArr[] = {"LIN","LOG"};
static ViString freqSweModeArr[] = {":SOUR:FREQ:MODE CW",":SOUR:FREQ:MODE SWE;:SOUR:SWE:MODE AUTO",":SOUR:FREQ:MODE SWE;:SOUR:SWE:MODE MAN",":SOUR:FREQ:MODE SWE;:SOUR:SWE:MODE STEP"};
static ViString powSweModeArr[] = {":SOUR:POW:MODE FIX",":SOUR:POW:MODE SWE;:SOUR:SWE:POW:MODE AUTO",":SOUR:POW:MODE SWE;:SOUR:SWE:POW:MODE MAN",":SOUR:POW:MODE SWE;:SOUR:SWE:POW:MODE STEP"};
static ViString phaseUnitArr[] = {"RAD","DEG"};                             
static ViString lstSeqArr[] = {":SOUR:LIST",":SYST:MSEQ"};
static ViString stpArr[] = {"*SAV","*RCL"};
static ViString stepModeArr[] = {"AUTO","STEP"};
static ViString trigSourceArr[] = {"AUTO","EXT","SING"};
static ViString trigSlopeArr[] = {"POS","NEG","EITH"};
static ViString trigAbortArr[] = {"SWE","LIST","MSEQ"};
static ViString VGHzScaleArr[] = {"1","0.5"};
static ViString sourceArr[] = {"INT","EXT"};
static ViString LFFreqModeArr[] = {"FIX","SWE"};
static ViString LFFreqInputModeArr[] = {"FIX","MAN"};
static ViString LFFunctionArr[] = {"SIN","SQU","TRI","PRN"};
static ViString LFSweModeArr[] = {":SOUR2:FREQ:MODE CW",":SOUR2:FREQ:MODE SWE;:SOUR2:SWE:MODE AUTO",":SOUR2:FREQ:MODE SWE;:SOUR2:SWE:MODE MAN",":SOUR2:FREQ:MODE SWE;:SOUR2:SWE:MODE STEP"};
static ViString stepConfigArr[] = {":SOUR:FREQ:STEP",":SOUR:FREQ:STEP",":SOUR:POW:STEP",":SOUR:POW:STEP"};
static ViString stepExeArr[] = {":SOUR:FREQ:MODE FIX;:SOUR:FREQ",":SOUR:FREQ:MODE SWE;:SOUR:FREQ",":SOUR:POW:MODE FIX;:SOUR:POW",":SOUR:POW:MODE SWE;:SOUR:POW"};
static ViString directArr[] = {"UP","DOWN"};
static ViString optionsList[] = {"SM-B1","SM-B2","SM-B2","SM-B5","SM-BX","SM-BX","SM-BX","SMP-B11","SMP-B12","SMP-B13","SMP-B14","SMP-B15","SMP-BX","SMP-B17","SMP-B18"};
static ViString instrVersionList[] = {"SMP02","SMP03","SMP04"};
static ViReal64 minLevelList[] = {-20.0, -130.0};
static ViReal64 maxLevelList[] = {27.0, 22.0, 22.0};
static ViReal64 minFrequencyList[] = {2.0E3, 10.0};
static ViReal64 maxFrequencyList[] = {20.0E3, 27.0E3, 40.0E3};
static ViString zeroOrTwo[] = {"0", "2"};
static ViString oneOrTwo[] = {"1", "2"};

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE  ============================*/
/*****************************************************************************/
/* rssmp_stringValPair is used in the rssmp_errorMessage function            */
/* rssmp_statusDataRanges is used to track session dependent status & ranges */
/* maxFrequency stores maximum frequency (dependent upon instrument version) */
/* minFrequency stores minimum frequency (dependent upon instrument option)  */    
/* option is used to store instrument options information                    */
/* buffer is used to write to and to read from the instrument                */
/*===========================================================================*/
static ViByte buffer[BUFFER_SIZE]; 

typedef struct  rssmp_stringValPair
{
   ViStatus stringVal;
   ViString stringName;
}  rssmp_tStringValPair;
 
struct rssmp_statusDataRanges {
    ViInt16 option;
    ViInt32 instr;
    ViBoolean firmware;
    ViReal64 minFrequency;    
    ViReal64 maxFrequency;
    ViReal64 minLevel;    
    ViReal64 maxLevel;
    ViChar instrDriverRevision[256];
};

typedef struct rssmp_statusDataRanges *rssmp_instrRange;

/*****************************************************************************/
/*= UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) =================*/
/*****************************************************************************/
ViBoolean rssmp_invalidViBooleanRange (ViBoolean val);
ViBoolean rssmp_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max);
ViBoolean rssmp_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean rssmp_invalidViUInt16Range (ViUInt16 val, ViUInt16 min, ViUInt16 max);
ViBoolean rssmp_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max);
ViBoolean rssmp_invalidViReal32Range (ViReal32 val, ViReal32 min, ViReal32 max);
ViBoolean rssmp_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max);
ViStatus rssmp_initCleanUp (ViSession openRMSession, ViPSession openInstrSession, ViStatus currentStatus);
ViStatus rssmp_readToFile (ViSession instrSession, ViString filename, ViUInt32 readBytes, ViPUInt32 retCount);
ViStatus rssmp_writeFromFile (ViSession instrSession, ViString filename, ViUInt32 writeBytes, ViPUInt32 retCount);

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT UTILITY ROUTINE DECLARATIONS =======================*/
/*****************************************************************************/
ViStatus rssmp_defaultInstrSetup (ViSession openInstrSession);
ViStatus rssmp_sysStatus (ViSession instrSession);
ViStatus rssmp_queryInstrOption (ViSession instrSession, ViInt16 queryBit, ViPInt16 bitValue);
ViStatus rssmp_confOPT (ViSession instrSession, ViString optsList);
ViStatus rssmp_queryInstrVersionFreq (ViSession instrSession, ViPReal64 minFreq, ViPReal64 maxFreq);
ViStatus rssmp_queryInstrVersionLev (ViSession instrSession, ViPReal64 minLev, ViPReal64 maxLev);

/*****************************************************************************/
/*= USER-CALLABLE FUNCTIONS (Exportable Functions) ==========================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function opens the instrument, queries the instrument      */
/*           for its ID, and initializes the instrument to a known state.    */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_init (ViRsrc resourceName, ViBoolean IDQuery,
                    ViBoolean reset, ViPSession instrSession)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViUInt16 interface = VI_INTF_GPIB;
    ViSession rmSession = 0;
    ViUInt32 retCnt = 0;
     

    /*- Check input parameter ranges ----------------------------------------*/

    if (rssmp_invalidViBooleanRange (IDQuery))
        return VI_ERROR_PARAMETER2;
    if (rssmp_invalidViBooleanRange (reset))
        return VI_ERROR_PARAMETER3;

    /*- Open instrument session ---------------------------------------------*/

    if ((rssmp_status = viOpenDefaultRM (&rmSession)) < 0)
        return rssmp_status;

    if ((rssmp_status = viOpen (rmSession, resourceName, VI_LOAD_CONFIG,
                                 VI_NULL, instrSession)) < 0)
    	{
        viClose (rmSession);
        return rssmp_status;
    	}

    /* Serial interface support */
    CHECKERR_CU( viGetAttribute (*instrSession, VI_ATTR_INTF_TYPE, &interface));

    if (interface == VI_INTF_ASRL)  /* Serial Port */
        {
        CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_TERMCHAR, '\n'));
        CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_TERMCHAR_EN,
                                     VI_TRUE));
        CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_ASRL_END_IN,
                                     VI_ASRL_END_TERMCHAR));
        CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_ASRL_END_OUT,
                                     VI_ASRL_END_TERMCHAR));

        /* Avoids incorrect behaviour of instrument after switched it ON. */
        if ((rssmp_status = viWrite (*instrSession, (ViBuf)"\r\n", 2, &retCnt)) < 0)
            return rssmp_initCleanUp (rmSession, instrSession, rssmp_status);

        }

    /*- Configure VISA Formatted I/O ----------------------------------------*/

    if ((rssmp_status = viSetAttribute (*instrSession, VI_ATTR_TMO_VALUE, 10000)) < 0)
            return rssmp_initCleanUp (rmSession, instrSession, rssmp_status);
    if ((rssmp_status = viSetBuf (*instrSession, VI_READ_BUF|VI_WRITE_BUF, 4000)) < 0)
            return rssmp_initCleanUp (rmSession, instrSession, rssmp_status);
    if ((rssmp_status = viSetAttribute (*instrSession, VI_ATTR_WR_BUF_OPER_MODE,
                            VI_FLUSH_ON_ACCESS)) < 0)
            return rssmp_initCleanUp (rmSession, instrSession, rssmp_status);
    if ((rssmp_status = viSetAttribute (*instrSession, VI_ATTR_RD_BUF_OPER_MODE,
                            VI_FLUSH_ON_ACCESS)) < 0)
            return rssmp_initCleanUp (rmSession, instrSession, rssmp_status);

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery)
    	{
        CHECKERR_CU( viQueryf (*instrSession, "*IDN?", "%s", buffer));
		if (!strstr (buffer, "Rohde&Schwarz,SMP") &&
			!strstr (buffer, "ROHDE&SCHWARZ,SMR"))
        	return rssmp_initCleanUp (rmSession, instrSession,
        	                           VI_ERROR_FAIL_ID_QUERY);
    	}

    /*- Reset instrument ----------------------------------------------------*/

    if (reset) {
        if ((rssmp_status = rssmp_reset (*instrSession)) < 0)
            return rssmp_initCleanUp (rmSession, instrSession, rssmp_status);
    }       
    else  /*- Send Default Instrument Setup ---------------------------------*/
        if ((rssmp_status = rssmp_defaultInstrSetup (*instrSession)) < 0)
            return rssmp_initCleanUp (rmSession, instrSession, rssmp_status);

    return rssmp_status;
}

/*===========================================================================*/

/*****************************************************************************/
/*= USER-CALLABLE INSTRUMENT-DEPENDENT ROUTINES =============================*/
/*****************************************************************************/
/*===========================================================================*/
/* Function: Output Amplitude Modulated Signal                               */
/* Purpose:  This function represents a high-level approach to the instrument*/
/* driver. It demonstrates how to create an application function which       */
/* contains a set of low-level functions. E.g. this function fully           */
/* configures the instrument to output an amplitude modulated signal.        */
/* This function performs the following operations:                          */
/* 1. Resets the instrument                                                  */
/* 2. Sets the RF output frequency                                           */
/* 3. Sets the RF output level                                               */
/* 4. Configures the ALC                                                     */
/* 5. Configures the internal LF generator (source of the modulating signal) */
/* 6. Configures the amplitude modulation of the RF output signal            */
/* 7. Turns on the instrument's RF output.                                   */      
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_applicOutpAmplModul (ViSession instrSession, ViReal64 carrierFreq, 
    ViReal64 carrierLevel, ViReal64 depth, ViBoolean LFSource, ViReal64 LFFreq, 
    ViInt16 LFShape)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViInt16 rssmp_optPresent = VI_FALSE;    
    ViReal64 rssmp_minFreq = 10.0E0;
    ViReal64 rssmp_maxFreq = 40.0E3;
    ViReal64 rssmp_minLev = -130.0;
    ViReal64 rssmp_maxLev = 27.0;
    ViReal64 junk = 0.0;
/*     
    if ((rssmp_status = rssmp_queryInstrVersionFreq (instrSession, &rssmp_minFreq, &rssmp_maxFreq)) < 0)
        return rssmp_status;     
    if ((rssmp_status = rssmp_queryInstrVersionLev (instrSession, &rssmp_minLev, &rssmp_maxLev)) < 0)
        return rssmp_status;     
*/
    if (rssmp_invalidViReal64Range (carrierFreq, rssmp_minFreq, rssmp_maxFreq))
        return VI_ERROR_PARAMETER2;
    if (rssmp_invalidViReal64Range (carrierLevel, rssmp_minLev, rssmp_maxLev))
        return VI_ERROR_PARAMETER3;
    if (rssmp_invalidViReal64Range (depth, 0.0, 100.0))
        return VI_ERROR_PARAMETER4;
    if (rssmp_invalidViBooleanRange (LFSource))        
        return VI_ERROR_PARAMETER5;
        
    if (LFSource == VI_FALSE) {
        if ((rssmp_status = rssmp_queryInstrOption (instrSession, 0x4, &rssmp_optPresent)) < 0)
            return rssmp_status;     
        if (rssmp_optPresent == VI_FALSE) {
            switch ((ViInt16)LFFreq) {
                case 400:
                case 1000:
                case 3000:
                case 15000:
                    if (modf (LFFreq, &junk) != 0)
                        return VI_ERROR_PARAMETER6;
                    break;
                default:
                    return VI_ERROR_PARAMETER6;
            }
            if (LFShape != 0)
                return VI_ERROR_PARAMETER7;
        }
        else {
            if (rssmp_invalidViReal64Range (LFFreq, 0.1, 5E5))        
                return VI_ERROR_PARAMETER6;        
            if (rssmp_invalidViInt16Range (LFShape, 0, 3))
                return VI_ERROR_PARAMETER7;        
        }
    }
    else {
        if ((rssmp_status = rssmp_queryInstrOption (instrSession, 0x2, &rssmp_optPresent)) < 0)
            return rssmp_status;     
        if (rssmp_optPresent == VI_FALSE) {
            rssmp_status = rssmp_VI_WARN_NSUP_OPTION;
            return rssmp_status;
        }
        else {
            if (rssmp_invalidViReal64Range (LFFreq, 0.1, 5E5))        
                return VI_ERROR_PARAMETER6;        
            if (rssmp_invalidViInt16Range (LFShape, 0, 3))
                return VI_ERROR_PARAMETER7;        
        }
    }   
    
    /*-------------- Reset ------------------------------------------------------*/ 
    
    if ((rssmp_status = rssmp_reset (instrSession)) < 0)
        return rssmp_status;                
        
    /*-------------- Configure Carrier Frequency --------------------------------*/ 
    
    if ((rssmp_status = rssmp_confCarrFreq (instrSession, carrierFreq, 0.0, 1.0, VI_FALSE)) < 0)
        return rssmp_status;                

    /*-------------- Configure Carrier Level ------------------------------------*/ 

    if ((rssmp_status = rssmp_confCarrLevel (instrSession, carrierLevel, 0.0, 16.0, 0, VI_FALSE, VI_FALSE)) < 0)
        return rssmp_status;                

    /*-------------- Configure Automatic Level Control --------------------------*/ 

    if ((rssmp_status = rssmp_confALC (instrSession, VI_TRUE, VI_FALSE, 2, 0, 1.0)) < 0)
        return rssmp_status;                

    /*-------------- Configure Internal Modulating Source (LF Generator) --------*/ 

    if (LFSource == VI_FALSE) {
        if ((rssmp_status = rssmp_confLFGen (instrSession, VI_FALSE, 1.0, LFFreq, LFShape, 1E3, 0, VI_FALSE)) < 0)              
            return rssmp_status;                
    }
    else
        if ((rssmp_status = rssmp_confLFGen (instrSession, VI_TRUE, 1.0, 1E3, 0, LFFreq, LFShape, VI_FALSE)) < 0)              
            return rssmp_status;                
        
    /*-------------- Configure Amplitude Modulation -----------------------------*/ 

    if ((rssmp_status = rssmp_confAmplModul (instrSession, VI_TRUE, LFSource, depth)) < 0)
        return rssmp_status;                
        
    /*-------------- Turn On The Instrument's Output ----------------------------*/         
        
    if ((rssmp_status = rssmp_outState (instrSession, VI_TRUE, VI_FALSE, VI_FALSE)) < 0)
        return rssmp_status;                
        
    return rssmp_status;
}

/*===========================================================================*/
/* Function: Frequency                                                       */
/* Purpose:  This function sets frequency of the instrument's RF output.     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_confCarrFreq (ViSession instrSession, ViReal64 freq,
    ViReal64 offset, ViReal64 mltplier, ViBoolean mode)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViReal64 rssmp_minFreq = 10.0E0;
    ViReal64 rssmp_maxFreq = 40.0E3;
/*    
    if ((rssmp_status = rssmp_queryInstrVersionFreq (instrSession, &rssmp_minFreq, &rssmp_maxFreq)) < 0)
        return rssmp_status;     
*/
    if (rssmp_invalidViReal64Range (offset, -50.0E3, 50.0E3))
        return VI_ERROR_PARAMETER3;
    if (rssmp_invalidViReal64Range (mltplier, 1.0, 10.0))
        return VI_ERROR_PARAMETER4;        
    if (rssmp_invalidViBooleanRange (mode))        
        return VI_ERROR_PARAMETER5;        
	if (offset == 0.0) /* check range only if offset is equal to 0.0 */
	    if (rssmp_invalidViReal64Range (freq, rssmp_minFreq, rssmp_maxFreq))
	        return VI_ERROR_PARAMETER2;
    
    if (mode == VI_TRUE) {
        if ((rssmp_status = viPrintf (instrSession, ":SOUR:FREQ:MODE SWE;:SOUR:FREQ:MAN %fMHZ", freq)) < 0)               
            return rssmp_status;       
    }           
    else
        if ((rssmp_status = viPrintf (instrSession, ":SOUR:FREQ:MODE CW;:SOUR:FREQ:CW %fMHZ;OFFS %fMHZ;MULT %f",
            freq, offset, mltplier)) < 0)                   
            return rssmp_status;       

    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
        return rssmp_status;                

    return rssmp_status;
}

/*===========================================================================*/
/* Function: Level                                                           */
/* Purpose:  This function controls the RF output level.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_confCarrLevel (ViSession instrSession, ViReal64 level, ViReal64 offset, 
    ViReal64 limit, ViInt16 unit, ViBoolean amplMode, ViBoolean attenuator)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViReal64 rssmp_minLev = -130.0;
    ViReal64 rssmp_maxLev = 27.0;
    rssmp_instrRange instrPtr;

    if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssmp_status;

/*    
    if ((rssmp_status = rssmp_queryInstrVersionLev (instrSession, &rssmp_minLev, &rssmp_maxLev)) < 0)
        return rssmp_status;     
*/
    if (rssmp_invalidViReal64Range (offset, -100.0, 100.0))
        return VI_ERROR_PARAMETER3;
	if ((instrPtr -> instr) == SMP) {
	    if (rssmp_invalidViInt16Range (unit, 0, 7))
	        return VI_ERROR_PARAMETER5;
	}
	else
		if (unit != 0)
			return VI_ERROR_PARAMETER5;
	
    if (rssmp_invalidViBooleanRange (amplMode))        
        return VI_ERROR_PARAMETER6;        
    if (rssmp_invalidViBooleanRange (attenuator))        
        return VI_ERROR_PARAMETER7;        
	if (offset == 0.0) /* check range only if offset is equal to 0.0 */
	    if (unit == 0) {
	        if (rssmp_invalidViReal64Range (level, rssmp_minLev, rssmp_maxLev))
	            return VI_ERROR_PARAMETER2;
	        if (rssmp_invalidViReal64Range (limit, rssmp_minLev, rssmp_maxLev))
	            return VI_ERROR_PARAMETER4;
	    }           

    if ((rssmp_status = viPrintf (instrSession, ":OUTP:AMOD %s", attrArr[attenuator])) < 0)               
        return rssmp_status;       

    if (amplMode == VI_TRUE) {
        if ((rssmp_status = viPrintf (instrSession, ":SOUR:POW:MODE SWE;:SOUR:POW %f%s", 
        level, unitArr[unit])) < 0)               
            return rssmp_status;       
    }
    else
        if ((rssmp_status = viPrintf (instrSession, ":SOUR:POW:MODE FIX;:SOUR:POW:OFFS %f;:SOUR:POW %f%s;:SOUR:POW:LIM %f%s", 
        offset, level, unitArr[unit], limit, unitArr[unit])) < 0)               
            return rssmp_status;       

    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
        return rssmp_status;                
        
    return rssmp_status;
}

/*===========================================================================*/
/* Function: ALC (Automatic Level Control)                                   */
/* Purpose:  This function enables and configures the automatic level control*/
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_confALC (ViSession instrSession, ViBoolean state, ViBoolean pMeter, 
    ViInt16 search, ViInt16 source, ViReal64 reference)
{
    ViStatus rssmp_status = VI_SUCCESS;
    rssmp_instrRange instrPtr;

    if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssmp_status;
    
    if (rssmp_invalidViBooleanRange (state))        
        return VI_ERROR_PARAMETER2;
    if (rssmp_invalidViBooleanRange (pMeter))        
        return VI_ERROR_PARAMETER3;         
    if (rssmp_invalidViInt16Range (search, 0, 2))
        return VI_ERROR_PARAMETER4;
    if (rssmp_invalidViInt16Range (source, 0, 2))
        return VI_ERROR_PARAMETER5;
    if (rssmp_invalidViReal64Range (reference, 0.0, 3.0))
        return VI_ERROR_PARAMETER6;
    
	pMeter =+ ((instrPtr -> instr) == SMP) ? 2 : 0;
    
    if ((rssmp_status = viPrintf (instrSession, ":SOUR:POW:ALC:SOUR %s;STAT %s;:SOUR:POW:ALC:SOUR:PMET %s",
        ALCSourceArr[source], stateArr[state],
        pMeterArr[pMeter])) < 0)
        return rssmp_status;       

    if (source)
        if ((rssmp_status = viPrintf (instrSession, ":POW:ALC:REF %fV", reference)) < 0)
            return rssmp_status;       
            
    if (state == VI_FALSE)
        if ((rssmp_status = viPrintf (instrSession, ":POW:ALC:SEAR %s", onceArr[search])) < 0)               
            return rssmp_status;       
                                        
    return rssmp_status;
}

/*===========================================================================*/
/* Function: User Correction                                                 */
/* Purpose:  This function is used to create and activate lists in which     */
/*           arbitrary RF frequencies are assigned level correction values.  */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_confUserCorr (ViSession instrSession, ViBoolean state, 
    ViInt16 action, ViString name, ViReal64 _VI_FAR freq[],
    ViReal64 _VI_FAR power[], ViInt32 length)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViInt32 i = 0;
    ViInt32 bytesFmt = 0;           
    ViUInt32 retCnt = 0;                                                 
    ViReal64 rssmp_minFreq = 10.0E0;
    ViReal64 rssmp_maxFreq = 40.0E3;
	rssmp_instrRange instrPtr;    

    if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssmp_status;
    
    if (rssmp_invalidViBooleanRange (state))        
        return VI_ERROR_PARAMETER2;         
    if (rssmp_invalidViInt16Range (action, 0, 2))
        return VI_ERROR_PARAMETER3;
    if (StringLength (name) > NAME_LEN) {
        rssmp_status = VI_ERROR_PARAMETER4;     
        return rssmp_status;
    }       
    if (rssmp_invalidViInt32Range (length, 1, 160))
        return VI_ERROR_PARAMETER7;
    
    if (state == VI_TRUE) {
/*
        if ((rssmp_status = rssmp_queryInstrVersionFreq (instrSession, &rssmp_minFreq, &rssmp_maxFreq)) < 0)
            return rssmp_status;
*/
        switch (action) {
            case 0:
			    if ((instrPtr -> firmware) == VI_TRUE) { /* VI_TRUE ...  SMR Firmware version 1.08 or higher */
	                if ((rssmp_status = viPrintf (instrSession, ":SOUR:CORR:CSET:SEL \"%s\"", name)) < 0)
	                    return rssmp_status;
			    }
			    else { /* VI_FALSE ... SMR Firmware version 1.07 */
	                if ((rssmp_status = viPrintf (instrSession, ":SOUR:CORR:CSET:SEL %s", name)) < 0)
	                    return rssmp_status;
			    }

                Fmt (buffer, "%s<:SOUR:CORR:CSET:DATA:FREQ ");
                bytesFmt = NumFmtdBytes ();             
            
                for (i = 0; i < length; i++) {
                    if (rssmp_invalidViReal64Range (freq[i], rssmp_minFreq, rssmp_maxFreq))
                        return VI_ERROR_PARAMETER5;
                    Fmt (buffer, "%s[a]<%fMHz,", freq[i]);     
                    bytesFmt += NumFmtdBytes ();            
                }

                if ((rssmp_status = viWrite (instrSession, buffer, bytesFmt - 1, &retCnt)) < 0)              
                    return rssmp_status;                
    
                Fmt (buffer, "%s<:SOUR:CORR:CSET:DATA:POW ");
                bytesFmt = NumFmtdBytes ();             
        
                for (i = 0; i < length; i++) {
                    if (rssmp_invalidViReal64Range (power[i], -6.0, 6.0))
                        return VI_ERROR_PARAMETER6;
                    Fmt (buffer, "%s[a]<%f,", power[i]);        
                    bytesFmt += NumFmtdBytes ();            
                }

                if ((rssmp_status = viWrite (instrSession, buffer, bytesFmt - 1, &retCnt)) < 0)              
                    return rssmp_status;                
                break;
            
            case 1:
			    if ((instrPtr -> firmware) == VI_TRUE) { /* VI_TRUE ...  SMR Firmware version 1.08 or higher */
	                if ((rssmp_status = viPrintf (instrSession, ":SOUR:CORR:CSET:DEL \"%s\"", name)) < 0)
	                    return rssmp_status;
			    }
			    else { /* VI_FALSE ... SMR Firmware version 1.07 */
	                if ((rssmp_status = viPrintf (instrSession, ":SOUR:CORR:CSET:DEL %s", name)) < 0)
	                    return rssmp_status;
			    }
                break;
            case 2:
			    if ((instrPtr -> firmware) == VI_TRUE) { /* VI_TRUE ...  SMR Firmware version 1.08 or higher */
        	        if ((rssmp_status = viPrintf (instrSession, ":SOUR:CORR:CSET:SEL \"%s\"", name)) < 0)
            	        return rssmp_status;
			    }
			    else { /* VI_FALSE ... SMR Firmware version 1.07 */
	                if ((rssmp_status = viPrintf (instrSession, ":SOUR:CORR:CSET:SEL %s", name)) < 0)
    	                return rssmp_status;
			    }
                break;
        }
    }
    
    if ((rssmp_status = viPrintf (instrSession, ":SOUR:CORR:STAT %s", stateArr[state])) < 0)               
        return rssmp_status;       
            
    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
        return rssmp_status;                
        
    return rssmp_status;
}

/*===========================================================================*/
/* Function: Amplitude Modulation                                            */
/* Purpose:  This function configures amplitude modulation.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_confAmplModul (ViSession instrSession, ViBoolean state,
    ViInt16 source, ViReal64 depth)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViInt16 rssmp_optPresent = VI_FALSE;
    rssmp_instrRange instrPtr;

    if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssmp_status;

    if (rssmp_invalidViBooleanRange (state))        
        return VI_ERROR_PARAMETER2;         
    if (rssmp_invalidViBooleanRange (source))        
        return VI_ERROR_PARAMETER3;         
    if (rssmp_invalidViReal64Range (depth, 0.0, 100.0))
        return VI_ERROR_PARAMETER4;

    if ((rssmp_status = rssmp_queryInstrOption (instrSession, 0x2, &rssmp_optPresent)) < 0)
        return rssmp_status;     
        
    if ((rssmp_optPresent == VI_FALSE) && (source == 1)) {
        rssmp_status = rssmp_VI_WARN_NSUP_OPTION;
        return rssmp_status;
    }
    
    if ((rssmp_status = viPrintf (instrSession, ":SOUR:AM:DEPT %f;SOUR %s;:SOUR:AM:STAT %s", 
        depth, 	((instrPtr -> instr) == SMP) ? AMSourceArr[source] : FMPMSourceArr[source],
        stateArr[state])) < 0)
        return rssmp_status;       

    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
        return rssmp_status;   
        
    return rssmp_status;       
}

/*===========================================================================*/
/* Function: Frequency Modulation                                            */
/* Purpose:  This function configures frequency modulation.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_confFreqModul (ViSession instrSession, ViBoolean state1, 
    ViBoolean state2, ViInt16 mode, ViReal64 deviation1, ViReal64 deviation2, 
    ViInt16 source1, ViInt16 source2)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViInt16 rssmp_optPresent = VI_FALSE;
    rssmp_instrRange instrPtr;

    if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssmp_status;

    if (rssmp_invalidViBooleanRange (state1))        
        return VI_ERROR_PARAMETER2;         
    if (rssmp_invalidViBooleanRange (state2))        
        return VI_ERROR_PARAMETER3;         
    if (rssmp_invalidViInt16Range (mode, 0, 2))
        return VI_ERROR_PARAMETER4;
    if (rssmp_invalidViReal64Range (deviation1, 0.0, 20E6))
        return VI_ERROR_PARAMETER5;
    if (rssmp_invalidViReal64Range (deviation2, 0.0, 20E6))
        return VI_ERROR_PARAMETER6;
    if (rssmp_invalidViInt16Range (source1, 0, 2))
        return VI_ERROR_PARAMETER7;
    if (rssmp_invalidViInt16Range (source2, 0, 2))
        return VI_ERROR_PARAMETER8;
    
    if ((rssmp_status = rssmp_queryInstrOption (instrSession, 0x8, &rssmp_optPresent)) < 0)
        return rssmp_status;     
    if (rssmp_optPresent == VI_FALSE) {
        if (mode == 2) {
            rssmp_status = rssmp_VI_WARN_NSUP_OPTION;
            return rssmp_status;
        }
    }       
    if ((rssmp_status = rssmp_queryInstrOption (instrSession, 0x2, &rssmp_optPresent)) < 0)
        return rssmp_status;     
    if (rssmp_optPresent == VI_FALSE) {
        if (source2 == 0) {
            rssmp_status = rssmp_VI_WARN_NSUP_OPTION;
            return rssmp_status;
        }
    }
    
	if ((instrPtr -> instr) == SMP) { /* SMP connected ? */
		if ((rssmp_status = viPrintf (instrSession, ":SOUR:FM1:SOUR %s;MODE %s;DEV %f;STAT %s;:SOUR:FM2:SOUR %s;DEV %f;STAT %s", 
	        FMPMSourceArr[source1], modulModeArr[mode], deviation1, stateArr[state1],
	        FMPMSourceArr[source2], deviation2, stateArr[state2])) < 0)               
	        return rssmp_status;
	}
	else { /* SMR connected */
		if ((rssmp_status = viPrintf (instrSession, ":SOUR:FM:SOUR %s;DEV %f;STAT %s", 
	        FMPMSourceArr[source1], deviation1, stateArr[state1])) < 0)               
	        return rssmp_status;
	}
    
    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
        return rssmp_status;   

    return rssmp_status;       
}

/*===========================================================================*/
/* Function: Phase Modulation                                                */
/* Purpose:  This function configures phase modulation.                      */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_confPhasModul (ViSession instrSession, ViBoolean state1,
    ViReal64 deviation1, ViBoolean state2, ViReal64 deviation2, ViBoolean unit,
    ViInt16 source1, ViInt16 source2)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViInt16 rssmp_optPresent = VI_FALSE;
    rssmp_instrRange instrPtr;

    if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssmp_status;

	if ((instrPtr -> instr) == SMP) { /* SMP connected ? */

	    if (rssmp_invalidViBooleanRange (unit))        
	        return VI_ERROR_PARAMETER6;         
	    if (rssmp_invalidViBooleanRange (state1))        
	        return VI_ERROR_PARAMETER2;         
	    if (rssmp_invalidViBooleanRange (state2))        
	        return VI_ERROR_PARAMETER3;         
	    if (unit == VI_FALSE) {
	        if (rssmp_invalidViReal64Range (deviation1, 0.0, 20.0))
	            return VI_ERROR_PARAMETER4;
	        if (rssmp_invalidViReal64Range (deviation2, 0.0, 20.0))
	            return VI_ERROR_PARAMETER5;
	    }
	    else {
	        if (rssmp_invalidViReal64Range (deviation1, 0.0, 20.0*180/PI))
	            return VI_ERROR_PARAMETER4;
	        if (rssmp_invalidViReal64Range (deviation2, 0.0, 20.0*180/PI))
	            return VI_ERROR_PARAMETER5;
	    }
	    if (rssmp_invalidViInt16Range (source1, 0, 2))
	        return VI_ERROR_PARAMETER7;
	    if (rssmp_invalidViInt16Range (source2, 0, 2))
	        return VI_ERROR_PARAMETER8;

	    if ((rssmp_status = rssmp_queryInstrOption (instrSession, 0x8, &rssmp_optPresent)) < 0)
	        return rssmp_status;     
	    if (rssmp_optPresent == VI_FALSE) {
	        if (state2 == VI_TRUE) {
	            rssmp_status = rssmp_VI_WARN_NSUP_OPTION;
	            return rssmp_status;
	        }
	    }
	    if ((rssmp_status = rssmp_queryInstrOption (instrSession, 0x2, &rssmp_optPresent)) < 0)
	        return rssmp_status;     
	    if (rssmp_optPresent == VI_FALSE) {
	        if (source2 == 0) {
	            rssmp_status = rssmp_VI_WARN_NSUP_OPTION;
	            return rssmp_status;
	        }
	    }
    
	    if ((rssmp_status = viPrintf (instrSession, ":SOUR:PM1:SOUR %s;DEV %f%s;STAT %s;:SOUR:PM2:SOUR %s;DEV %f%s;STAT %s", 
	        FMPMSourceArr[source1], deviation1, phaseUnitArr[unit], stateArr[state1],
	        FMPMSourceArr[source2], deviation2, phaseUnitArr[unit], stateArr[state2])) < 0)
	        return rssmp_status;
    
	    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
	        return rssmp_status;   
	}
	else
		return rssmp_VI_WARN_NSUP_FUNCTION; /* SMR is not currently supporting this function */

    return rssmp_status;       
}

/*===========================================================================*/
/* Function: Pulse Modulation                                                */
/* Purpose:  This function configures pulse modulation.                      */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_confPulsModul (ViSession instrSession, ViBoolean state,
    ViBoolean polarity, ViBoolean extImp, ViReal64 pulseWidth, ViReal64 pulseDelay,
    ViReal64 pulsePerFreq, ViBoolean pulseParam, ViBoolean pulseShape, ViBoolean source)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViInt16 rssmp_optPresent = VI_FALSE;
    rssmp_instrRange instrPtr;

    if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssmp_status;

    if (rssmp_invalidViBooleanRange (state))        
        return VI_ERROR_PARAMETER2;         
    if (rssmp_invalidViBooleanRange (polarity))        
        return VI_ERROR_PARAMETER3;         
    if (rssmp_invalidViBooleanRange (extImp))        
        return VI_ERROR_PARAMETER4;         
    if (rssmp_invalidViReal64Range (pulseWidth, 20E-9, 1.0))        
        return VI_ERROR_PARAMETER5;         
    if (rssmp_invalidViReal64Range (pulseDelay, 20E-9, 1.0))        
        return VI_ERROR_PARAMETER6;         
	if ((instrPtr -> instr) == SMP) { /* SMP connected ? */    
	    if (rssmp_invalidViBooleanRange (pulseParam))        
	        return VI_ERROR_PARAMETER8;
	}
	else { /* SMR connected */
		if (pulseParam != 0)
			return VI_ERROR_PARAMETER8;
	}								  
    if (pulseParam == VI_FALSE) {
        if (rssmp_invalidViReal64Range (pulsePerFreq, 100E-9, 85.0))        
            return VI_ERROR_PARAMETER7;         
    }
    else
        if (rssmp_invalidViReal64Range (pulsePerFreq, 0.01176, 10E6))        
            return VI_ERROR_PARAMETER7;         
    if (rssmp_invalidViBooleanRange (pulseShape))        
        return VI_ERROR_INSTR_PARAMETER9;         
    if (rssmp_invalidViBooleanRange (source))        
        return VI_ERROR_INSTR_PARAMETER10;        
        
    if ((rssmp_status = rssmp_queryInstrOption (instrSession, 0x400, &rssmp_optPresent)) < 0)
        return rssmp_status;     

    if (rssmp_optPresent == VI_FALSE) {
        if ((extImp == VI_FALSE) || (source == VI_FALSE)) {
            rssmp_status = rssmp_VI_WARN_NSUP_OPTION;
            return rssmp_status;
        }
        if ((rssmp_status = viPrintf (instrSession, ":SOUR:PULM:POL %s;SOUR EXT;STAT %s", 
                polarityArr[polarity], stateArr[state])) < 0)               
                return rssmp_status;        
    }
    else {
        if ((rssmp_status = viPrintf (instrSession, ":SOUR:PULM:EXT:IMP %s"
        	";:SOUR:PULM:POL %s;SOUR %s;:SOUR:PULS:DEL %f;WIDT %f"
        	";:SOUR:PULS:DOUB:STAT %s;%s %f;:SOUR:PULM:STAT %s", 
                pulMExtImpArr[extImp], polarityArr[polarity], sourceArr[source], 
                pulseDelay, pulseWidth, stateArr[pulseShape], pulsePerFreqArr[pulseParam], 
                pulsePerFreq, stateArr[state])) < 0)               
                return rssmp_status;        
    }           

    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
        return rssmp_status;   

    return rssmp_status;       
}

/*===========================================================================*/
/* Function: Digital Modulation                                              */
/* Purpose:  This function configures digital FSK and ASK modulations.       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_confDigitModul (ViSession instrSession, ViBoolean modType,
    ViBoolean state, ViBoolean polarity, ViReal64 FSKDev, ViInt16 FSKMode,
    ViReal64 ASKDepth)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViInt16 rssmp_optPresent = VI_FALSE;
    rssmp_instrRange instrPtr;

    if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssmp_status;

    if (rssmp_invalidViBooleanRange (modType))        
        return VI_ERROR_PARAMETER2;         
    if (rssmp_invalidViBooleanRange (state))        
        return VI_ERROR_PARAMETER3;         
    if (rssmp_invalidViBooleanRange (polarity))        
        return VI_ERROR_PARAMETER4;         
    if (rssmp_invalidViReal64Range (FSKDev, 0.0, 20E6))        
        return VI_ERROR_PARAMETER5;         
    if (rssmp_invalidViInt16Range (FSKMode, 0, 2))        
        return VI_ERROR_PARAMETER6;         
    if (rssmp_invalidViReal64Range (ASKDepth, 0.0, 100.0))        
        return VI_ERROR_PARAMETER7;         
        
    if ((rssmp_status = rssmp_queryInstrOption (instrSession, 0x8, &rssmp_optPresent)) < 0)
        return rssmp_status;     
    
    if (rssmp_optPresent == VI_FALSE) 
        if (modType == VI_FALSE) {
            rssmp_status = rssmp_VI_WARN_NSUP_OPTION;
            return rssmp_status;
        }       
        
	if ((instrPtr -> instr) == SMP) { /* SMP connected ? */
	    if ((rssmp_status = viPrintf (instrSession, ":SOUR:DM:TYPE %s;STAT %s;ASK:POL %s;DEPT %f;"
	    	":SOUR:DM:FSK:MODE %s;DEV %f", DMTypeArr[modType], stateArr[state],
	    	polarityArr[polarity], ASKDepth, modulModeArr[FSKMode], FSKDev)) < 0)               
	        return rssmp_status;       
	}
	else { /* SMR connected */
	    if ((rssmp_status = viPrintf (instrSession, ":SOUR:DM:TYPE %s;STAT %s;ASK:POL %s;DEPT %f;"
	    	":SOUR:DM:FSK:POL %s;DEV %f", DMTypeArr[modType], stateArr[state],
	    	polarityArr[polarity], ASKDepth, polarityArr[polarity], FSKDev)) < 0)               
	        return rssmp_status;       
	}

    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
        return rssmp_status;   

    return rssmp_status;       
}

/*===========================================================================*/
/* Function: External Modulation Inputs                                      */
/* Purpose:  This function configures external modulation inputs.            */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_confExtInput (ViSession instrSession, ViBoolean ext1Coupl,
    ViBoolean ext2Coupl, ViBoolean ext1Imped, ViBoolean ext2Imped)
{
    ViStatus rssmp_status = VI_SUCCESS;
    
    if (rssmp_invalidViBooleanRange (ext1Coupl))        
        return VI_ERROR_PARAMETER2;         
    if (rssmp_invalidViBooleanRange (ext2Coupl))        
        return VI_ERROR_PARAMETER3;         
    if (rssmp_invalidViBooleanRange (ext1Imped))        
        return VI_ERROR_PARAMETER4;         
    if (rssmp_invalidViBooleanRange (ext2Imped))        
        return VI_ERROR_PARAMETER5;         

    if ((rssmp_status = viPrintf (instrSession, ":SOUR:FM:EXT1:IMP %s;COUP %s;:SOUR:FM:EXT2:IMP %s;COUP %s", 
        extImpedanceArr[ext1Imped], couplingArr[ext1Coupl], extImpedanceArr[ext2Imped], couplingArr[ext2Coupl])) < 0)               
        return rssmp_status;       

    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
        return rssmp_status;                
        
    return rssmp_status;       
}

/*===========================================================================*/
/* Function: Frequency Sweep                                                 */
/* Purpose:  This function configures frequency sweep of the RF output.      */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_confFreqSweep (ViSession instrSession, ViInt16 mode,
    ViInt16 freqInp, ViReal64 startFreq, ViReal64 stopFreq, ViReal64 centerFreq,
    ViReal64 spanFreq, ViReal64 dwell, ViReal64 step, ViBoolean spacing)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViReal64 rssmp_minFreq = 10.0E0;
    ViReal64 rssmp_maxFreq = 40.0E3;

    if (rssmp_invalidViInt16Range (mode, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (rssmp_invalidViInt16Range (freqInp, 0, 3))
        return VI_ERROR_PARAMETER3;
    if (rssmp_invalidViReal64Range (dwell, 1E-3, 1.0))
        return VI_ERROR_PARAMETER8;        
    if (rssmp_invalidViBooleanRange (spacing))        
        return VI_ERROR_INSTR_PARAMETER10;        
    if (spacing == VI_TRUE) {
        if (rssmp_invalidViReal64Range (step, 0.01, 50.0))
            return VI_ERROR_INSTR_PARAMETER9;        
    }           
    else
        if (rssmp_invalidViReal64Range (step, 0.0, 1E9))
            return VI_ERROR_INSTR_PARAMETER9;        
        
/*
    if ((rssmp_status = rssmp_queryInstrVersionFreq (instrSession, &rssmp_minFreq, &rssmp_maxFreq)) < 0)
        return rssmp_status;     
*/

    switch (freqInp) {
        case 1:
            startFreq = centerFreq - spanFreq / 2;
            stopFreq = centerFreq + spanFreq / 2;
            if (rssmp_invalidViReal64Range (centerFreq, rssmp_minFreq, rssmp_maxFreq))
                return VI_ERROR_PARAMETER6;
            if (rssmp_invalidViReal64Range (startFreq, rssmp_minFreq, rssmp_maxFreq))
                return VI_ERROR_PARAMETER7;
            if (rssmp_invalidViReal64Range (stopFreq, rssmp_minFreq, rssmp_maxFreq))
                return VI_ERROR_PARAMETER7;
        break;
        case 2:
            stopFreq = startFreq + spanFreq;
            if (rssmp_invalidViReal64Range (startFreq, rssmp_minFreq, rssmp_maxFreq))
                return VI_ERROR_PARAMETER4;
            if (rssmp_invalidViReal64Range (stopFreq, rssmp_minFreq, rssmp_maxFreq))
                return VI_ERROR_PARAMETER7;
        break;
        case 3:
            startFreq = stopFreq - spanFreq;
                if (rssmp_invalidViReal64Range (startFreq, rssmp_minFreq, rssmp_maxFreq))
                    return VI_ERROR_PARAMETER5;
                if (rssmp_invalidViReal64Range (stopFreq, rssmp_minFreq, rssmp_maxFreq))
                    return VI_ERROR_PARAMETER7;
        break;
        default:
                if (rssmp_invalidViReal64Range (startFreq, rssmp_minFreq, rssmp_maxFreq))
                    return VI_ERROR_PARAMETER4;
                if (rssmp_invalidViReal64Range (stopFreq, rssmp_minFreq, rssmp_maxFreq))
                    return VI_ERROR_PARAMETER5;
        break;
    }
    
    if ((rssmp_status = viPrintf (instrSession, ":SOUR:FREQ:STAR %fMHZ;STOP %fMHZ;:SOUR:SWE:DWEL %fS;:SOUR:SWE:FREQ:SPAC %s;STEP:%s %f;%s", 
        startFreq, stopFreq, dwell, spacingArr[spacing], spacingArr[spacing], step, 
        freqSweModeArr[mode])) < 0)               
        return rssmp_status;       

    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
        return rssmp_status;   

    return rssmp_status;       
}

/*===========================================================================*/
/* Function: Frequency Sweep Markers                                         */
/* Purpose:  This function enables and configures frequency sweep markers.   */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_confFreqSweepMrks (ViSession instrSession, ViReal64 freqM1,
    ViReal64 freqM2, ViReal64 freqM3, ViBoolean stateM1, ViBoolean stateM2,
    ViBoolean stateM3, ViBoolean amplM1, ViBoolean amplM2, ViBoolean amplM3, ViBoolean polarity)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViReal64 rssmp_minFreq = 10.0E0;
    ViReal64 rssmp_maxFreq = 40.0E3;
    rssmp_instrRange instrPtr;

    if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssmp_status;

    if (rssmp_invalidViBooleanRange (stateM1))        
        return VI_ERROR_PARAMETER5;        
    if (rssmp_invalidViBooleanRange (stateM2))        
        return VI_ERROR_PARAMETER6;        
    if (rssmp_invalidViBooleanRange (stateM3))        
        return VI_ERROR_PARAMETER7;        
    if (rssmp_invalidViBooleanRange (amplM1))        
        return VI_ERROR_PARAMETER8;        
    if (rssmp_invalidViBooleanRange (amplM2))        
        return VI_ERROR_INSTR_PARAMETER9;        
    if (rssmp_invalidViBooleanRange (amplM3))        
        return VI_ERROR_INSTR_PARAMETER10;        
    if (rssmp_invalidViBooleanRange (polarity))        
        return VI_ERROR_INSTR_PARAMETER11;        

/*
    if ((rssmp_status = rssmp_queryInstrVersionFreq (instrSession, &rssmp_minFreq, &rssmp_maxFreq)) < 0)
        return rssmp_status;     
*/

    if (rssmp_invalidViReal64Range (freqM1, rssmp_minFreq, rssmp_maxFreq))
        return VI_ERROR_PARAMETER2;
    if (rssmp_invalidViReal64Range (freqM2, rssmp_minFreq, rssmp_maxFreq))
        return VI_ERROR_PARAMETER3;
    if (rssmp_invalidViReal64Range (freqM3, rssmp_minFreq, rssmp_maxFreq))
        return VI_ERROR_PARAMETER4;

	if ((instrPtr -> instr) == SMP) { /* SMP connected ? */
		if ((rssmp_status = viPrintf (instrSession, ":SOUR:MARK1:AMPL %s;FREQ %fMHZ;STAT %s"
	    	";:SOUR:MARK2:AMPL %s;FREQ %fMHZ;STAT %s;:SOUR:MARK3:AMPL %s;FREQ %fMHZ;STAT %s"
	    	";:SOUR:MARK:POL %s", stateArr[amplM1], freqM1, stateArr[stateM1],
	    	stateArr[amplM2], freqM2, stateArr[stateM1], stateArr[amplM3], freqM3,
	    	stateArr[stateM3], polarityArr[polarity])) < 0)
	        return rssmp_status;       
	}
	else { /* SMR connected */
		if ((rssmp_status = viPrintf (instrSession, ":SOUR:MARK1:AMPL %s;FREQ %fMHZ;STAT %s"
	    	";:SOUR:MARK2:AMPL %s;FREQ %fMHZ;STAT %s;:SOUR:MARK3:AMPL %s;FREQ %fMHZ;STAT %s",
	    	stateArr[amplM1], freqM1, stateArr[stateM1],
	    	stateArr[amplM2], freqM2, stateArr[stateM1], stateArr[amplM3], freqM3,
	    	stateArr[stateM3])) < 0)
	        return rssmp_status;       
	}

    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
        return rssmp_status;   

    return rssmp_status;       
}

/*===========================================================================*/
/* Function: Level Sweep                                                     */
/* Purpose:  This function configures level sweep of the RF output.          */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_confLevelSweep (ViSession instrSession, ViInt16 mode,
    ViReal64 startLev, ViReal64 stopLev, ViInt16 unit, ViReal64 dwell, ViReal64 step)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViReal64 rssmp_minLev = -130.0;
    ViReal64 rssmp_maxLev = 27.0;
/*    
    if ((rssmp_status = rssmp_queryInstrVersionLev (instrSession, &rssmp_minLev, &rssmp_maxLev)) < 0)
        return rssmp_status;     
*/    
    if (rssmp_invalidViInt16Range (mode, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (rssmp_invalidViInt16Range (unit, 0, 7))
        return VI_ERROR_PARAMETER5;        
    if (rssmp_invalidViReal64Range (dwell, 1E-3, 1.0))
        return VI_ERROR_PARAMETER6;        
    if (rssmp_invalidViReal64Range (step, 0.1, 10.0))
        return VI_ERROR_PARAMETER7;        
        
    if (unit == 0) {
        if (rssmp_invalidViReal64Range (startLev, rssmp_minLev, rssmp_maxLev))
            return VI_ERROR_PARAMETER3;
        if (rssmp_invalidViReal64Range (stopLev, rssmp_minLev, rssmp_maxLev))
            return VI_ERROR_PARAMETER4;
    }           

    if ((rssmp_status = viPrintf (instrSession, ":SOUR:POW:STAR %f%s;STOP %f%s;:SOUR:SWE:POW:DWEL %f;STEP:LOG %f;%s",
        startLev, unitArr[unit], stopLev, unitArr[unit], dwell, step, powSweModeArr[mode])) < 0)               
        return rssmp_status;       

    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
        return rssmp_status;                
        
    return rssmp_status;
}

/*===========================================================================*/
/* Function: Level Sweep Markers                                             */
/* Purpose:  This function enables and configures level sweep markers.       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_confLevelSweepMrks (ViSession instrSession, ViReal64 levelM1,
    ViReal64 levelM2, ViReal64 levelM3, ViBoolean stateM1, ViBoolean stateM2,
    ViBoolean stateM3, ViBoolean polarity)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViReal64 rssmp_minLev = -130.0;
    ViReal64 rssmp_maxLev = 27.0;
    rssmp_instrRange instrPtr;

    if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssmp_status;

/*    
    if ((rssmp_status = rssmp_queryInstrVersionLev (instrSession, &rssmp_minLev, &rssmp_maxLev)) < 0)
        return rssmp_status;     
*/
    if (rssmp_invalidViReal64Range (levelM1, rssmp_minLev, rssmp_maxLev))
        return VI_ERROR_PARAMETER2;
    if (rssmp_invalidViReal64Range (levelM2, rssmp_minLev, rssmp_maxLev))
        return VI_ERROR_PARAMETER3;
    if (rssmp_invalidViReal64Range (levelM3, rssmp_minLev, rssmp_maxLev))
        return VI_ERROR_PARAMETER4;
    if (rssmp_invalidViBooleanRange (stateM1))        
        return VI_ERROR_PARAMETER5;        
    if (rssmp_invalidViBooleanRange (stateM2))        
        return VI_ERROR_PARAMETER6;        
    if (rssmp_invalidViBooleanRange (stateM3))        
        return VI_ERROR_PARAMETER7;        
    if (rssmp_invalidViBooleanRange (polarity))        
        return VI_ERROR_PARAMETER8;        
    
	if ((instrPtr -> instr) == SMP) { /* SMP connected ? */
	    if ((rssmp_status = viPrintf (instrSession, ":SOUR:MARK1:PSW:POW %f;STAT %s"
	    	";:SOUR:MARK2:PSW:POW %f;STAT %s;:SOUR:MARK3:PSW:POW %f;STAT %s;:SOUR:MARK:POL %s",
	        levelM1, stateArr[stateM1], levelM2, stateArr[stateM2], levelM3, stateArr[stateM3],
	        polarityArr[polarity])) < 0)              
	        return rssmp_status;       
	}
	else {
	    if ((rssmp_status = viPrintf (instrSession, ":SOUR:MARK1:PSW:POW %f;STAT %s"
	    	";:SOUR:MARK2:PSW:POW %f;STAT %s;:SOUR:MARK3:PSW:POW %f;STAT %s",
	        levelM1, stateArr[stateM1], levelM2, stateArr[stateM2], levelM3, stateArr[stateM3])) < 0)
	        return rssmp_status;       
	}
	
    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
        return rssmp_status;                
        
    return rssmp_status;
}               
                
/*=========================================================================*/
/* Function: LF Generator                                                  */
/* Purpose:  This function selects the internal LF generator and configures*/
/*           its parameters.                                               */
/*=========================================================================*/
ViStatus _VI_FUNC rssmp_confLFGen (ViSession instrSession, ViBoolean source,
    ViReal64 volt, ViReal64 freq1, ViInt16 shape1, ViReal64 freq2,  ViInt16 shape2, 
    ViBoolean LF2mode)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViInt16 rssmp_optPresent = VI_FALSE;
    ViReal64 junk = 0.0;    
    rssmp_instrRange instrPtr;

    if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssmp_status;

    if (rssmp_invalidViBooleanRange (source))        
        return VI_ERROR_PARAMETER2;        
    if (rssmp_invalidViReal64Range (volt, 0.0, 4.0))        
        return VI_ERROR_PARAMETER3;        
    if (rssmp_invalidViInt16Range (shape1, 0, 3))        
        return VI_ERROR_PARAMETER5;        
    if (rssmp_invalidViReal64Range (freq2, 0.1, 5E5))        
        return VI_ERROR_PARAMETER6;        
    if (rssmp_invalidViInt16Range (shape2, 0, 3))        
        return VI_ERROR_PARAMETER7;        
    if (rssmp_invalidViBooleanRange (LF2mode))        
        return VI_ERROR_PARAMETER8;        
        
	if ((instrPtr -> instr) == SMP) { /* SMP connected ? */
	    if (source == VI_TRUE) {
	        if ((rssmp_status = rssmp_queryInstrOption (instrSession, 0x2, &rssmp_optPresent)) < 0)
	            return rssmp_status;     
	        if (rssmp_optPresent == VI_FALSE) {
	            rssmp_status = rssmp_VI_WARN_NSUP_OPTION;
	            return rssmp_status;
	        }
	        if ((rssmp_status = viPrintf (instrSession, ":SOUR2:FREQ:MODE %s;:SOUR2:FREQ:%s %f;:SOUR2:FUNC %s;:OUTP2:VOLT %f;SOUR 2",
	            LFFreqModeArr[LF2mode], LFFreqInputModeArr[LF2mode], freq2, LFFunctionArr[shape2], volt)) < 0)
	            return rssmp_status;       
	    }
	    else {
	        if ((rssmp_status = rssmp_queryInstrOption (instrSession, 0x4, &rssmp_optPresent)) < 0)
	            return rssmp_status;     
	        if (rssmp_optPresent == VI_TRUE) {
	            if (rssmp_invalidViReal64Range (freq1, 0.1, 5E5))        
	                return VI_ERROR_PARAMETER4;        
	            if ((rssmp_status = viPrintf (instrSession, ":SOUR0:FREQ %f;:SOUR0:FUNC %s;:OUTP2:VOLT %f;SOUR 0",
	                freq1, LFFunctionArr[shape1], volt)) < 0)
	                return rssmp_status;       
	        }
	        else {
	            if (rssmp_optPresent == VI_FALSE) {
	                switch ((ViInt16)freq1) {
	                    case 400:
	                    case 1000:
	                    case 3000:
	                    case 15000:
	                        if (modf (freq1, &junk) != 0)
	                            return VI_ERROR_PARAMETER4;
	                        break;
	                    default:
	                        return VI_ERROR_PARAMETER4;
	                }
	                if (shape1 != 0)
	                    return rssmp_VI_WARN_NSUP_OPTION;
	            }
	            if ((rssmp_status = viPrintf (instrSession, ":SOUR0:FREQ %f;:OUTP2:SOUR 0", freq1)) < 0)
	                return rssmp_status;       
	        }           
	    }
	}
	else { /* SMR connected */
	    if (rssmp_invalidViInt16Range (shape2, 0, 1))
	        return VI_ERROR_PARAMETER7;        
        
        if ((rssmp_status = viPrintf (instrSession, ":SOUR2:FREQ:MODE %s;:SOUR2:FREQ:%s %f;:SOUR2:FUNC:SHAP %s"
        	";:OUTP2:VOLT %fV", LFFreqModeArr[LF2mode], LFFreqInputModeArr[LF2mode], freq2,
        	LFFunctionArr[shape2], volt)) < 0)
            return rssmp_status;       
	}
    
    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
        return rssmp_status;                
        
    return rssmp_status;
}

/*=========================================================================*/
/* Function: LF Sweep                                                      */
/* Purpose:  This function configures frequency sweep of the internal      */
/*           LF generator.                                                 */
/*=========================================================================*/
ViStatus _VI_FUNC rssmp_confLFSweep (ViSession instrSession, ViInt16 mode,
    ViReal64 startFreq, ViReal64 stopFreq, ViBoolean spacing, ViReal64 step, 
    ViReal64 dwell)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViInt16 rssmp_optPresent = VI_FALSE;
            
    if (rssmp_invalidViInt16Range (mode, 0, 3))        
        return VI_ERROR_PARAMETER2;        
    if (rssmp_invalidViReal64Range (startFreq, 0.1, 5E5))        
        return VI_ERROR_PARAMETER3;        
    if (rssmp_invalidViReal64Range (stopFreq, 0.1, 5E5))        
        return VI_ERROR_PARAMETER4;        
    if (rssmp_invalidViBooleanRange (spacing))        
        return VI_ERROR_PARAMETER5;       
    if (spacing == VI_TRUE)   {   
        if (rssmp_invalidViReal64Range (step, 0.01, 50.0))        
            return VI_ERROR_PARAMETER6;        
    }           
    else
        if (rssmp_invalidViReal64Range (step, 0.0, 5E5))        
            return VI_ERROR_PARAMETER6;        
    if (rssmp_invalidViReal64Range (dwell, 1E-3, 1.0))        
        return VI_ERROR_PARAMETER7;        

    if ((rssmp_status = rssmp_queryInstrOption (instrSession, 0x2, &rssmp_optPresent)) < 0)
        return rssmp_status;     
        
    if (rssmp_optPresent == VI_TRUE) {
        if ((rssmp_status = viPrintf (instrSession, ":SOUR2:FREQ:STAR %f;STOP %f;:SOUR2:SWE:DWEL %f;SPAC %s;STEP:%s %f;%s", 
            startFreq, stopFreq, dwell, spacingArr[spacing], spacingArr[spacing],
            step, LFSweModeArr[mode])) < 0)
            return rssmp_status;       
    }
    else {
        rssmp_status = rssmp_VI_WARN_NSUP_OPTION;
        return rssmp_status;
    }           
        
    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
        return rssmp_status;                
        
    return rssmp_status;
}

/*=========================================================================*/
/* Function: LF Sweep Markers                                              */
/* Purpose:  This function enables and configures frequency sweep markers  */
/*           of the internal LF generator.                                 */
/*=========================================================================*/
ViStatus _VI_FUNC rssmp_confLFSweepMrks (ViSession instrSession, ViReal64 freqM1,
    ViReal64 freqM2, ViReal64 freqM3, ViBoolean stateM1, ViBoolean stateM2,
    ViBoolean stateM3, ViBoolean polarity)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViInt16 rssmp_optPresent = VI_FALSE;

    if (rssmp_invalidViReal64Range (freqM1, 0.1, 5E5))
        return VI_ERROR_PARAMETER2;
    if (rssmp_invalidViReal64Range (freqM2, 0.1, 5E5))
        return VI_ERROR_PARAMETER3;
    if (rssmp_invalidViReal64Range (freqM3, 0.1, 5E5))
        return VI_ERROR_PARAMETER4;
    if (rssmp_invalidViBooleanRange (stateM1))        
        return VI_ERROR_PARAMETER5;        
    if (rssmp_invalidViBooleanRange (stateM2))        
        return VI_ERROR_PARAMETER6;        
    if (rssmp_invalidViBooleanRange (stateM3))        
        return VI_ERROR_PARAMETER7;        
    if (rssmp_invalidViBooleanRange (polarity))        
        return VI_ERROR_PARAMETER8;        

    if ((rssmp_status = rssmp_queryInstrOption (instrSession, 0x4, &rssmp_optPresent)) < 0)
        return rssmp_status;     

    if (rssmp_optPresent == VI_TRUE) {
        if ((rssmp_status = viPrintf (instrSession, ":SOUR2:MARK1:FREQ %f;STAT %s"
        	";:SOUR2:MARK2:FREQ %f;STAT %s;:SOUR2:MARK3:FREQ %f;STAT %s;:SOUR2:MARK:POL %s", 
            freqM1, stateArr[stateM1], freqM2, stateArr[stateM2], freqM3, stateArr[stateM3], 
            polarityArr[polarity])) < 0)               
            return rssmp_status;       
    }           
    else {
        rssmp_status = rssmp_VI_WARN_NSUP_OPTION;
        return rssmp_status;
    }           

    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
        return rssmp_status;   

    return rssmp_status;       
}

/*===========================================================================*/
/* Function: RF Frequency                                                    */
/* Purpose:  This function sets the frequency of the RF output signal.       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_qRFfrequency (ViSession instrSession,
                    ViReal64 freq, ViBoolean checkRange)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];
    ViReal64 rssmp_minFreq = 10.0E0;
    ViReal64 rssmp_maxFreq = 40.0E3;
    
    if (checkRange) {
        /*- Check input parameter ranges ------------------------------------*/
/*
        if ((rssmp_status = rssmp_queryInstrVersionFreq (instrSession, &rssmp_minFreq, &rssmp_maxFreq)) < 0)
            return rssmp_status;
*/
        if (rssmp_invalidViReal64Range (freq, rssmp_minFreq, rssmp_maxFreq))
            return VI_ERROR_PARAMETER2;
    }
    Fmt (buffer, "%s<:SOUR:FREQ:CW %f[p2]", 1.0e6*freq);   
    if ((rssmp_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rssmp_status;

    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
        return rssmp_status;                

    return rssmp_status;
}

/*===========================================================================*/
/* Function: RF Level                                                        */
/* Purpose:  This function sets the level of the RF output signal.           */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_qRFlevel (ViSession instrSession,
                    ViReal64 level, ViBoolean checkRange)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];
    ViReal64 rssmp_minLev = -130.0;
    ViReal64 rssmp_maxLev = 27.0;
    
    if (checkRange) {    
        /*- Check input parameter ranges ------------------------------------*/
/*
        if ((rssmp_status = rssmp_queryInstrVersionLev (instrSession, &rssmp_minLev, &rssmp_maxLev)) < 0)
            return rssmp_status;
*/
        if (rssmp_invalidViReal64Range (level, rssmp_minLev, rssmp_maxLev))
            return VI_ERROR_PARAMETER2;
    }     

    Fmt (buffer, "%s<:POW %f[p2]", level);   
    if ((rssmp_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rssmp_status;

    return rssmp_status;
}

/*===========================================================================*/
/* Function: RF Level Offset                                                 */
/* Purpose:  This function sets the level offset of the RF output signal.    */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_qRFlevOffset (ViSession instrSession,
                    ViReal64 offset, ViBoolean checkRange)

{
    ViStatus rssmp_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];
    
    if (checkRange) {    
        /*- Check input parameter ranges ------------------------------------*/
        if (rssmp_invalidViReal64Range (offset, -100.0, 100.0))
            return VI_ERROR_PARAMETER2;
    }
    
    Fmt (buffer, "%s<:POW:OFFS %f[p2]", offset);   
    if ((rssmp_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rssmp_status;

    return rssmp_status;
}

/*===========================================================================*/
/* Function: RF Start Stop Frequency                                         */
/* Purpose:  This function sets the start and stop frequencies of the RF     */
/*           sweep.                                                          */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_qRFstartStopFreq (ViSession instrSession,
                    ViReal64 startFreq, ViReal64 stopFreq, ViBoolean checkRange)                                  
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];
    ViReal64 rssmp_minFreq = 10.0E0;
    ViReal64 rssmp_maxFreq = 40.0E3;
    
    if (checkRange) {
        /*- Check input parameter ranges ------------------------------------*/
/*
        if ((rssmp_status = rssmp_queryInstrVersionFreq (instrSession, &rssmp_minFreq,
            &rssmp_maxFreq)) < 0)
            return rssmp_status;
*/
        if (rssmp_invalidViReal64Range (startFreq, rssmp_minFreq, rssmp_maxFreq))
            return VI_ERROR_PARAMETER2;
        if (rssmp_invalidViReal64Range (stopFreq, rssmp_minFreq, rssmp_maxFreq))
            return VI_ERROR_PARAMETER3;
    }

    Fmt (buffer, "%s<:FREQ:STAR %f[p2];STOP %f[p2]", 1.0e6*startFreq,
        1.0e6*stopFreq);
    if ((rssmp_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rssmp_status;

    return rssmp_status;
}

/*===========================================================================*/
/* Function: RF Center Span Frequency                                        */
/* Purpose:  This function sets the center and span frequencies of the RF    */
/*           sweep.                                                          */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_qRFcenterSpanFreq (ViSession instrSession,
                    ViReal64 centerFreq, ViReal64 spanFreq, ViBoolean checkRange)                                  
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];
    ViReal64 rssmp_minFreq = 10.0E0;
    ViReal64 rssmp_maxFreq = 40.0E3;
    
    if (checkRange) {
        /*- Check input parameter ranges ------------------------------------*/
/*
		if ((rssmp_status = rssmp_queryInstrVersionFreq (instrSession, &rssmp_minFreq,
            &rssmp_maxFreq)) < 0)
            return rssmp_status;
*/
        if (rssmp_invalidViReal64Range (centerFreq, rssmp_minFreq, rssmp_maxFreq))
            return VI_ERROR_PARAMETER2;
        if (rssmp_invalidViReal64Range (spanFreq, rssmp_minFreq, rssmp_maxFreq))
            return VI_ERROR_PARAMETER3;
    }
    
    Fmt (buffer, "%s<:FREQ:CENT %f[p2];SPAN %f[p2]", 1.0e6*centerFreq,
        1.0e6*spanFreq);
    if ((rssmp_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rssmp_status;

    return rssmp_status;
}

/*===========================================================================*/
/* Function: RF Sweep Step                                                   */
/* Purpose:  This function sets the RF sweep step in MHz.                    */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_qRFsweepStep (ViSession instrSession,
                    ViReal64 step, ViBoolean checkRange)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];

    if (checkRange) {
        if (rssmp_invalidViReal64Range (step, 0.0, 1000.0))
            return VI_ERROR_PARAMETER2;
    }

    Fmt (buffer, "%s<:SWE:STEP %f[p2]", 1.0e6*step);
    if ((rssmp_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rssmp_status;

    return rssmp_status;
}
                                  
/*===========================================================================*/
/* Function: LF Frequency                                                    */
/* Purpose:  This function sets the LF output frequency.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_qLFfreq (ViSession instrSession, ViBoolean source,
                    ViReal64 freq, ViBoolean checkRange)                           
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];
    ViInt16 rssmp_optPresent = VI_FALSE;
    ViReal64 junk = 0.0;
    rssmp_instrRange instrPtr;

    if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssmp_status;

	if ((instrPtr -> instr) == SMP) { /* SMP connected ? */
	
	    if (checkRange) {    
	        /*- Check input parameter ranges ------------------------------------*/
	        if (rssmp_invalidViBooleanRange (source))        
	            return VI_ERROR_PARAMETER2;
	        if (rssmp_invalidViReal64Range (freq, 0.1, 5E5))
	            return VI_ERROR_PARAMETER3;
        
	        if ((rssmp_status = rssmp_queryInstrOption (instrSession, 0x2, &rssmp_optPresent)) < 0)
	            return rssmp_status;     
            
	        if (source) {
	            // INT2
	            if (rssmp_optPresent == VI_FALSE) {
	                rssmp_status = rssmp_VI_WARN_NSUP_OPTION;
	                return rssmp_status;
	            }
	        }
	        else {
	            // INT1
	            if (rssmp_optPresent == VI_FALSE) {
	                switch ((ViInt16)freq) {
	                    case 400:
	                    case 1000:
	                    case 3000:
	                    case 15000:
	                        if (modf (freq, &junk) != 0)
	                            return VI_ERROR_PARAMETER3;
	                        break;
	                    default:
	                        return VI_ERROR_PARAMETER3;
	                }
	            }
	        }
	    }

	    Fmt (buffer, ":SOUR%s:FREQ %f[p2]", zeroOrTwo[source], freq);
	    if ((rssmp_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
	        return rssmp_status;

	}
	else { /* SMR connected */
	    if ((rssmp_status = viPrintf (instrSession, ":SOUR2:FREQ %f", freq)) < 0)
	        return rssmp_status;
	}
	
    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
        return rssmp_status;                

    return rssmp_status;
}

/*===========================================================================*/
/* Function: LF Voltage                                                      */
/* Purpose:  This function sets the LF output voltage.                       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_qLFvolt (ViSession instrSession, ViReal64 volt,
                    ViBoolean checkRange)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];
    
    if (checkRange) {   
        /*- Check input parameter ranges ------------------------------------*/
        if (rssmp_invalidViReal64Range (volt, 0.0, 4.0))        
            return VI_ERROR_PARAMETER2;        
    }
    
    Fmt (buffer, "%s<:OUTP2:VOLT %f[p3]V", volt);
    if ((rssmp_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rssmp_status;

    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
        return rssmp_status;                

    return rssmp_status;
}
                         
/*===========================================================================*/
/* Function: AM Modulation Depth                                             */
/* Purpose:  This function sets the AM modulation depth.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_qAMmodDep (ViSession instrSession,
                    ViReal64 modDepth, ViBoolean checkRange)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];
    
    if (checkRange) {    
        /*- Check input parameter ranges ------------------------------------*/
        if (rssmp_invalidViReal64Range (modDepth, 0.0, 100.0))
            return VI_ERROR_PARAMETER2;
    }
    
    Fmt (buffer, "%s<:AM %f[p2]", modDepth);
    if ((rssmp_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rssmp_status;

    return rssmp_status;
}

/*===========================================================================*/
/* Function: FM Deviation                                                    */
/* Purpose:  This function sets the FM Deviation.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_qFMdev (ViSession instrSession, ViBoolean modulator,
                    ViReal64 freqDev, ViBoolean checkRange)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];

    if (checkRange) {    
        if (rssmp_invalidViBooleanRange (modulator))
            return VI_ERROR_PARAMETER2;
        if (rssmp_invalidViReal64Range (freqDev, 0.0, 20.0E6))
            return VI_ERROR_PARAMETER3;
    }
    
    Fmt (buffer, "%s<:FM%s %f[p2]", oneOrTwo[modulator], freqDev);

    if ((rssmp_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rssmp_status;

    return rssmp_status;
}

/*===========================================================================*/
/* Function: Phase Mod Deviation                                             */
/* Purpose:  This function sets the PHM Deviation.                           */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_qPHMdev (ViSession instrSession, ViBoolean modulator,
                    ViReal64 phaseDev, ViBoolean checkRange)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];
    rssmp_instrRange instrPtr;

    if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssmp_status;

	if ((instrPtr -> instr) == SMP) { /* SMP connected ? */
	    if (checkRange) {    
	        if (rssmp_invalidViBooleanRange (modulator))
	            return VI_ERROR_PARAMETER2;
	        if (rssmp_invalidViReal64Range (phaseDev, 0.0, 20.0))
	            return VI_ERROR_PARAMETER3;
	    }
    
	    Fmt (buffer, "%s<:PM%s %f[p3]", oneOrTwo[modulator], phaseDev);

	    if ((rssmp_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
	        return rssmp_status;
	}
	else /* SMR connected */
		return rssmp_VI_WARN_NSUP_FUNCTION;
		
    return rssmp_status;
}

/*===========================================================================*/
/* Function: Pulse Modulator Delay                                           */
/* Purpose:  This function sets the pulse modulation delay.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_qPulsModDelay (ViSession instrSession, ViReal64 delay,
                    ViBoolean checkRange)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];
    ViInt16 rssmp_optPresent = VI_FALSE;
    
    if (checkRange) {    
        if ((rssmp_status = rssmp_queryInstrOption (instrSession, 0x400, &rssmp_optPresent)) < 0)
            return rssmp_status;     
        if (rssmp_optPresent == VI_FALSE) {
            rssmp_status = rssmp_VI_WARN_NSUP_OPTION;
            return rssmp_status;
        }
        if (rssmp_invalidViReal64Range (delay, 20.0E-9, 1.0))
            return VI_ERROR_PARAMETER2;
    }
    
    Fmt (buffer, "%s<:PULS:DEL %f", delay);
    if ((rssmp_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rssmp_status;

    return rssmp_status;
}

/*===========================================================================*/
/* Function: Pulse Modulator Period                                          */
/* Purpose:  This function sets the pulse modulation period.                 */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_qPulsModPeriod (ViSession instrSession, ViReal64 period,
                    ViBoolean checkRange)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];
    ViInt16 rssmp_optPresent = VI_FALSE;
    
    if (checkRange) {    
        if ((rssmp_status = rssmp_queryInstrOption (instrSession, 0x400, &rssmp_optPresent)) < 0)
            return rssmp_status;     
        if (rssmp_optPresent == VI_FALSE) {
            rssmp_status = rssmp_VI_WARN_NSUP_OPTION;
            return rssmp_status;
        }
        if (rssmp_invalidViReal64Range (period, 100.0E-9, 85.0))
            return VI_ERROR_PARAMETER2;
    }
    
    Fmt (buffer, "%s<:PULS:PER %f", period);
    if ((rssmp_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rssmp_status;

    return rssmp_status;
}

/*===========================================================================*/
/* Function: Pulse Modulator Width                                           */
/* Purpose:  This function sets the pulse modulation width.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_qPulsModWidth (ViSession instrSession, ViReal64 width,
                    ViBoolean checkRange)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];
    ViInt16 rssmp_optPresent = VI_FALSE;
    
    if (checkRange) {    
        if ((rssmp_status = rssmp_queryInstrOption (instrSession, 0x400, &rssmp_optPresent)) < 0)
            return rssmp_status;     
        if (rssmp_optPresent == VI_FALSE) {
            rssmp_status = rssmp_VI_WARN_NSUP_OPTION;
            return rssmp_status;
        }
        if (rssmp_invalidViReal64Range (width, 20.0E-9, 1.0))
            return VI_ERROR_PARAMETER2;
    }
    
    Fmt (buffer, "%s<:PULS:WIDT %f", width);
    if ((rssmp_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rssmp_status;

    return rssmp_status;
}

/*=========================================================================*/
/* Function: Save/Recall Setup                                             */
/* Purpose:  This function saves and recalls instrument's setup.           */
/*=========================================================================*/
ViStatus _VI_FUNC rssmp_confSaveRecallSetup (ViSession instrSession, ViInt32 stpNumber,
    ViBoolean setup)
{
    ViStatus rssmp_status = VI_SUCCESS;                       

    if (rssmp_invalidViBooleanRange (setup))        
        return VI_ERROR_PARAMETER3;         
    if (setup == VI_FALSE) {
        if (rssmp_invalidViInt32Range (stpNumber, 1, 50))        
            return VI_ERROR_PARAMETER2;         
    }
    else
        if (rssmp_invalidViInt32Range (stpNumber, 0, 50))        
            return VI_ERROR_PARAMETER2;         
        
    if ((rssmp_status = viPrintf (instrSession, "%s %ld", stpArr[setup], stpNumber)) < 0)              
        return rssmp_status;       
        
    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
        return rssmp_status;                
        
    return rssmp_status;
}

/*===========================================================================*/
/* Function: Edit Catalog                                                    */
/* Purpose:  This function allows the user to edit catalog of lists          */
/*           and memory sequences stored in the instrument's memory.         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_confEditCatalog (ViSession instrSession, ViInt16 action,
                                         ViChar _VI_FAR oldCatalog[],
                                         ViPInt32 oldOccupMem,
                                         ViPInt32 oldVacantMem,
                                         ViString listName,
                                         ViChar _VI_FAR newCatalog[],
                                         ViPInt32 newOccupMem,
                                         ViPInt32 newVacantMem, ViBoolean setup)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;                      
    rssmp_instrRange instrPtr;

    if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssmp_status;

    if (rssmp_invalidViInt16Range (action, 0, 3))        
        return VI_ERROR_PARAMETER2;         
    if (rssmp_invalidViBooleanRange (setup))        
        return VI_ERROR_PARAMETER3;         
    if (StringLength (listName) > 7) {
        return VI_ERROR_PARAMETER6;
    }       

    if ((rssmp_status = viPrintf (instrSession, "%s:CAT?", lstSeqArr[setup])) < 0)
        return rssmp_status;
    if ((rssmp_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rssmp_status;
            
    if (Scan (buffer, "%s>%s", oldCatalog) != 1) {
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    }

    oldCatalog[retCnt - 1] = 0x0;
    
    if ((rssmp_status = viPrintf (instrSession, "%s:FREE?",lstSeqArr[setup])) < 0)
        return rssmp_status;
    if ((rssmp_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rssmp_status;
            
    if (Scan (buffer, "%s>%i[b4],%i[b4]", oldVacantMem, oldOccupMem) != 2) {
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    }       

    if (action > 0) {
        switch (action) {
            case (1):
	            if ((rssmp_status = viPrintf (instrSession, "%s:SEL \"%s\";*WAI", 
	                lstSeqArr[setup], listName)) < 0)              
	                return rssmp_status;
            break;
            case (2):
				if ((instrPtr -> instr) == SMP) { /* SMP connected ? */
	                if ((rssmp_status = viPrintf (instrSession, "%s:DEL \"%s\";*WAI", 
	                    lstSeqArr[setup], listName)) < 0)              
	                    return rssmp_status;
	            }
	            else { /* SMR connected */
				    if ((instrPtr -> firmware) == VI_TRUE) { /* VI_TRUE ... SMR Firmware version 1.08 or higher */
		                if ((rssmp_status = viPrintf (instrSession, "%s:DEL \"%s\";*WAI", 
		                    lstSeqArr[setup], listName)) < 0)              
		                    return rssmp_status;
				    }
				    else { /* VI_FALSE ... SMR Firmware version 1.07 */
		                if ((rssmp_status = viPrintf (instrSession, "%s:DEL %s;*WAI", 
		                    lstSeqArr[setup], listName)) < 0)              
		                    return rssmp_status;
				    }
	            }
            break;
            case (3):
                if ((rssmp_status = viPrintf (instrSession, ":SOUR:FREQ:MODE CW;*WAI;%s:DEL:ALL;*WAI",
                    lstSeqArr[setup])) < 0)              
                    return rssmp_status;       
            break;
        }           
        
        if ((rssmp_status = viPrintf (instrSession, "%s:CAT?", lstSeqArr[setup])) < 0)
            return rssmp_status;
        if ((rssmp_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
            return rssmp_status;
            
        if (Scan (buffer, "%s>%s", newCatalog) != 1) {
            rssmp_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            return rssmp_status;
        }

        newCatalog[retCnt - 1] = 0x0;

        if ((rssmp_status = viPrintf (instrSession, "%s:FREE?", lstSeqArr[setup])) < 0)
            return rssmp_status;
        if ((rssmp_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
            return rssmp_status;
            
        if (Scan (buffer, "%s>%i[b4],%i[b4]", newVacantMem, newOccupMem) != 2) {
            rssmp_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;      
            return rssmp_status;
        }       
                        
    }
    else {
        Scan (oldCatalog, "%s>%s", newCatalog);
        *newVacantMem = *oldVacantMem;
        *newOccupMem = *oldOccupMem;
    }       
    
    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
        return rssmp_status;                
        
    return rssmp_status;
}

/*===========================================================================*/
/* Function: Create List                                                     */
/* Purpose:  This function creates a new list.                               */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_confCreateLst (ViSession instrSession, ViString listName,
    ViInt16 listLen, ViReal64 _VI_FAR freq_arr[], ViReal64 _VI_FAR level_arr[], ViReal64 dwell)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViReal64 rssmp_minFreq = 10.0E0;
    ViReal64 rssmp_maxFreq = 40.0E3;
    ViUInt32 retCnt = 0;    
    ViInt16 bytesFmt = 0;
    ViInt32 maxTmo = 15;
    ViInt32 i = 0;
    rssmp_instrRange instrPtr;

    if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssmp_status;

	if ((instrPtr -> instr) == SMP) { /* SMP connected ? */
    
	    if (StringLength (listName) > NAME_LEN) {
	        rssmp_status = VI_ERROR_PARAMETER2;     
	        return rssmp_status;
	    }       

	    if (rssmp_invalidViInt16Range (listLen, 0, 2000))        
	        return VI_ERROR_PARAMETER3;         
		/*

	    if ((rssmp_status = rssmp_queryInstrVersionFreq (instrSession, &rssmp_minFreq, &rssmp_maxFreq)) < 0)
	        return rssmp_status;

		*/
		
	    for (i=0; i<listLen; i++) {
	        if (rssmp_invalidViReal64Range (freq_arr[i], rssmp_minFreq * 1.0E6, rssmp_maxFreq * 1.0E6))
	            return VI_ERROR_PARAMETER4;
	        if (rssmp_invalidViReal64Range (level_arr[i], -144.0, +16.0))
	            return VI_ERROR_PARAMETER5;
	    }    

	    if (rssmp_invalidViReal64Range (dwell, 0.001, 1.0))
	       return VI_ERROR_PARAMETER6;

	    if ((rssmp_status = viPrintf (instrSession, ":SOUR:LIST:SEL \"%s\";:FORM:DATA PACK;BORD NORM",
	        listName)) < 0)              
	        return rssmp_status;       

	    Fmt (buffer, "%s<%d[b2]", listLen * BYTE_LEN);
	    bytesFmt = NumFmtdBytes();
    
	    Fmt (buffer, "%s<:SOUR:LIST:FREQ #%d[b2]%d[b2]" , bytesFmt, listLen * BYTE_LEN);
	    bytesFmt = NumFmtdBytes ();

	    Fmt (buffer, "%*f[zi*]<%*f", listLen, bytesFmt, listLen, freq_arr);
	    bytesFmt += NumFmtdBytes ();    
    
	    if ((rssmp_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)              
	        return rssmp_status;      
        
	    Fmt (buffer, "%s<%d[b2]", listLen * BYTE_LEN);
	    bytesFmt = NumFmtdBytes();
    
	    Fmt (buffer, "%s<:SOUR:LIST:POW #%d[b2]%d[b2]" , bytesFmt, listLen * BYTE_LEN);
	    bytesFmt = NumFmtdBytes ();

	    Fmt (buffer, "%*f[zi*]<%*f", listLen, bytesFmt, listLen, level_arr);
	    bytesFmt += NumFmtdBytes ();    
    
	    if ((rssmp_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)              
	        return rssmp_status;                
        
	    if ((rssmp_status = viPrintf (instrSession, ":SOUR:LIST:DWEL %f;*WAI", dwell)) < 0)
	        return rssmp_status;       

	    if ((rssmp_status = viPrintf (instrSession, ":SOUR:LIST:LEAR")) < 0)
	        return rssmp_status;       
        
	    if ((rssmp_operationComplete (instrSession, maxTmo))  < 0)              
	        return rssmp_status;       
	}
	else { /* SMR connected */

	    if (StringLength (listName) > NAME_LEN) {
	        rssmp_status = VI_ERROR_PARAMETER2;     
	        return rssmp_status;
	    }       
	    if (rssmp_invalidViInt16Range (listLen, 1, 10))
	        return VI_ERROR_PARAMETER3;         
	    if (rssmp_invalidViReal64Range (dwell, 0.001, 1.0))
	       return VI_ERROR_PARAMETER6;

		if ((rssmp_status = viPrintf (instrSession, ":SOUR:LIST:SEL \"%s\";DWEL %Lf;FREQ %,*Lf;POW %,*Lf",
			listName, dwell, listLen, freq_arr, listLen, level_arr)) < 0)
	        return rssmp_status;
	}
    
    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
        return rssmp_status;                
        
    return rssmp_status;
}

/*=========================================================================*/
/* Function: Create Memory Sequence                                        */
/* Purpose:  This function creates a new memory sequence.                  */
/*=========================================================================*/
ViStatus _VI_FUNC rssmp_confCreateMemSeq (ViSession instrSession, ViString seqName,
    ViInt32 length, ViInt32 _VI_FAR stpList[], ViReal64 _VI_FAR dwellList[])
{
    ViStatus rssmp_status = VI_SUCCESS;                       
    ViInt32 i = 0;
    ViInt32 bytesFmt = 0;           
    ViUInt32 retCnt = 0;                                                 
    rssmp_instrRange instrPtr;

    if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssmp_status;

    if (rssmp_invalidViInt32Range (length, 1, 256))        
        return VI_ERROR_PARAMETER2;         
    if (StringLength (seqName) > NAME_LEN) {
        rssmp_status = VI_ERROR_PARAMETER3;     
        return rssmp_status;
    }       

	if ((instrPtr -> instr) == SMP) { /* SMP connected ? */
	
	    if ((rssmp_status = viPrintf (instrSession, ":SYST:MSEQ:SEL \"%s\"", seqName)) < 0)              
	        return rssmp_status;       

	    Fmt (buffer, "%s<:SYST:MSEQ:RCL ");
	    bytesFmt = NumFmtdBytes ();             
    
	    for (i = 0; i < length; i++) {
	        if (rssmp_invalidViInt32Range (stpList[i], 1, 256))        
	            return VI_ERROR_PARAMETER4;         
	        Fmt (buffer, "%s[a]<%d[b4],", stpList[i]);      
	        bytesFmt += NumFmtdBytes ();            
	    }

	    if ((rssmp_status = viWrite (instrSession, buffer, bytesFmt - 1, &retCnt)) < 0)              
	        return rssmp_status;                

	    Fmt (buffer, "%s<:SYST:MSEQ:DWEL ");
	    bytesFmt = NumFmtdBytes ();             
    
	    for (i = 0; i < length; i++) {
	        if (rssmp_invalidViReal64Range (dwellList[i], 0.05, 60.0))        
	            return VI_ERROR_PARAMETER5;         
	        Fmt (buffer, "%s[a]<%f[p3],", dwellList[i]);        
	        bytesFmt += NumFmtdBytes ();            
	    }

	    if ((rssmp_status = viWrite (instrSession, buffer, bytesFmt - 1, &retCnt)) < 0)
	        return rssmp_status;
	}
	else { /* SMR connected */
	    if ((instrPtr -> firmware) == VI_TRUE) { /* VI_TRUE ... SMR Firmware version 1.08 or higher */
			if ((rssmp_status = viPrintf (instrSession, ":SYST:MSEQ:SEL \"%s\"",
				seqName)) < 0)
		        return rssmp_status;
	    }
	    else { /* VI_FALSE ... SMR Firmware version 1.07 */
			if ((rssmp_status = viPrintf (instrSession, ":SYST:MSEQ:SEL %s",
				seqName)) < 0)
		        return rssmp_status;
	    }
		if ((rssmp_status = viPrintf (instrSession, ":SYST:MSEQ:RCL %,*Ld",
			length, stpList)) < 0)
	        return rssmp_status;
		if ((rssmp_status = viPrintf (instrSession, ":SYST:MSEQ:DWEL %,*Lf",
			length, dwellList)) < 0)
	        return rssmp_status;
	}

    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
        return rssmp_status;                
        
    return rssmp_status;
}

/*=========================================================================*/
/* Function: Select List/Sequence                                          */
/* Purpose:  This function activates a selected list or memory sequence.   */
/*=========================================================================*/
ViStatus _VI_FUNC rssmp_confSelLstSeq (ViSession instrSession, ViInt16 setupMode, 
    ViBoolean stepMode, ViString name)
{
    ViStatus rssmp_status = VI_SUCCESS;                       
    ViInt32 maxTmo = 15;
    
    if (rssmp_invalidViInt16Range (setupMode, 0, 2))        
        return VI_ERROR_PARAMETER2;         
    if (rssmp_invalidViBooleanRange (stepMode))        
        return VI_ERROR_PARAMETER3;         
    if (StringLength (name) > NAME_LEN) {
        rssmp_status = VI_ERROR_PARAMETER4;     
        return rssmp_status;
    }       
    
    if (setupMode == 0) {
        if ((rssmp_status = viPrintf (instrSession, ":SOUR:FREQ:MODE FIX;:SYST:MODE FIX")) < 0)
            return rssmp_status;
    }       
    else {          
        if (setupMode == 1) {
            if ((rssmp_status = viPrintf (instrSession, ":SYST:MODE FIX;:SOUR:FREQ:MODE FIX;:SOUR:LIST:SEL \"%s\"",
                name)) < 0)                     
                return rssmp_status;                                        
            if ((rssmp_status = viPrintf (instrSession, ":SOUR:LIST:LEAR")) < 0)              
                return rssmp_status;       
            if ((rssmp_operationComplete (instrSession, maxTmo))  < 0)              
                return rssmp_status;       
            if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
                return rssmp_status;                
            if ((rssmp_status = viPrintf (instrSession, ":SOUR:LIST:MODE %s;:SOUR:FREQ:MODE LIST",
                 stepModeArr[stepMode])) < 0)              
                return rssmp_status;       
        }
        else 
            if ((rssmp_status = viPrintf (instrSession, ":SOUR:FREQ:MODE FIX;:SYST:MODE MSEQ;:SYST:MSEQ:MODE %s;:SYST:MSEQ:SEL \"%s\"",
                stepModeArr[stepMode], name)) < 0)              
                return rssmp_status;       
    }
    
    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
        return rssmp_status;                
        
    return rssmp_status;
}

/*=========================================================================*/
/* Function: Trigger                                                       */
/* Purpose:  This function configures the trigger system.                  */
/*=========================================================================*/
ViStatus _VI_FUNC rssmp_confTrigger (ViSession instrSession, ViInt16 trigger,
    ViInt16 source, ViInt16 trigSlope, ViBoolean abortPol)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViInt16 rssmp_optPresent = VI_FALSE;
    ViInt32 bytesFmt = 0;           
    ViUInt32 retCnt = 0;                                                 
	rssmp_instrRange instrPtr;    

    if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssmp_status;

    if (rssmp_invalidViInt16Range (trigger, 0, 4))        
        return VI_ERROR_PARAMETER2;        
    if (trigger == 4) {
        if (rssmp_invalidViInt16Range (source, 0, 1))        
            return VI_ERROR_PARAMETER3;        
        if (rssmp_invalidViInt16Range (trigSlope, 0, 1))        
            return VI_ERROR_PARAMETER4;        
    }
    else {
        if (rssmp_invalidViInt16Range (source, 0, 2))        
            return VI_ERROR_PARAMETER3;        
        if (rssmp_invalidViInt16Range (trigSlope, 0, 2))        
            return VI_ERROR_PARAMETER4;        
    }
    if (rssmp_invalidViBooleanRange (abortPol))        
        return VI_ERROR_PARAMETER5;        
    
	if ((instrPtr -> instr) == SMP) { /* SMP connected ? */
	    switch (trigger) {
	        case 0:
	            Fmt (buffer, "%s<:TRIG1:SWE:SOUR %s;:TRIG1:SLOP %s;:INP:POL %s", trigSourceArr[source],
	                trigSlopeArr[trigSlope], polarityArr[abortPol]);
	            bytesFmt = NumFmtdBytes ();                             
	            break;
	        case 1:
	            if ((rssmp_status = rssmp_queryInstrOption (instrSession, 0x4, &rssmp_optPresent)) < 0)
	                return rssmp_status;     
	            if (rssmp_optPresent == VI_TRUE) {
	                Fmt (buffer, "%s<:TRIG2:SWE:SOUR %s;:TRIG2:SLOP %s;:INP:POL %s", trigSourceArr[source],
	                    trigSlopeArr[trigSlope], polarityArr[abortPol]);
	                bytesFmt = NumFmtdBytes ();     
	            }               
	            else {
	                rssmp_status = rssmp_VI_WARN_NSUP_OPTION;
	                return rssmp_status;
	            }
	            break;
	        case 2:
	            Fmt (buffer, "%s<:TRIG:LIST:SOUR %s;:TRIG:SLOP %s;:INP:POL %s", trigSourceArr[source],
	                trigSlopeArr[trigSlope], polarityArr[abortPol]);
	            bytesFmt = NumFmtdBytes ();                                             
	            break;
	        case 3:
	            Fmt (buffer, "%s<:TRIG:MSEQ:SOUR %s;:TRIG:SLOP %s;:INP:POL %s", trigSourceArr[source],
	                trigSlopeArr[trigSlope], polarityArr[abortPol]);
	            bytesFmt = NumFmtdBytes ();                                             
	            break;
	        case 4:
	            if ((rssmp_status = rssmp_queryInstrOption (instrSession, 0x400, &rssmp_optPresent)) < 0)
	                return rssmp_status;     
	            if (rssmp_optPresent == VI_TRUE) {            
	                Fmt (buffer, "%s<:TRIG:PULS:SOUR %s;:TRIG:PULS:SLOP %s;:INP:POL %s", trigSourceArr[source],
	                    trigSlopeArr[trigSlope], polarityArr[abortPol]);
	                bytesFmt = NumFmtdBytes ();                             
	            }               
	            else {
	                rssmp_status = rssmp_VI_WARN_NSUP_OPTION;
	                return rssmp_status;
	            }
	            break;
	    }

	    if ((rssmp_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)              
	        return rssmp_status;                
	}
	else { /* SMR connected */
	    switch (trigger) {
	        case 0:
	            Fmt (buffer, "%s<:TRIG1:SWE:SOUR %s;:TRIG1:SLOP %s",
	            	trigSourceArr[source], trigSlopeArr[trigSlope]);
	            bytesFmt = NumFmtdBytes ();                             
	            break;
	        case 1:
                Fmt (buffer, "%s<:TRIG2:SWE:SOUR %s;:TRIG2:SLOP %s",
                	trigSourceArr[source], trigSlopeArr[trigSlope]);
                bytesFmt = NumFmtdBytes ();     
	            break;
	        case 2:
	            Fmt (buffer, "%s<:TRIG:LIST:SOUR %s;:TRIG:SLOP %s",
	            	trigSourceArr[source], trigSlopeArr[trigSlope]);
	            bytesFmt = NumFmtdBytes ();                                             
	            break;
	        case 3:
	            Fmt (buffer, "%s<:TRIG:MSEQ:SOUR %s;:TRIG:SLOP %s",
	                trigSourceArr[source], trigSlopeArr[trigSlope]);
	            bytesFmt = NumFmtdBytes ();                                             
	            break;
	        case 4:
                Fmt (buffer, "%s<:TRIG:PULS:SOUR %s;:TRIG:PULS:SLOP %s",
                	trigSourceArr[source], trigSlopeArr[trigSlope]);
                bytesFmt = NumFmtdBytes ();                             
	    }

	    if ((rssmp_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)              
	        return rssmp_status;                
	}
    
    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
        return rssmp_status;                
        
    return rssmp_status;
}

/*===========================================================================*/
/* Function: Instrument Options                                              */
/* Purpose:  This function returns the instrument version and list of        */
/*           options which the instrument is equipped with.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_confOPTLst (ViSession instrSession, ViString optsList,
				ViString instrVer)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViInt16 i = 0;
    rssmp_instrRange instrPtr;

    if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssmp_status;

	if ((instrPtr -> instr) == SMP) { /* SMP connected ? */
	    FillBytes ((ViChar *)buffer, 0, BUFFER_SIZE, '\0');
    
	    if ((rssmp_status = viWrite (instrSession, (ViBuf)"*IDN?", 5, &retCnt)) < 0)
	        return rssmp_status;
	    if ((rssmp_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
	        return rssmp_status;

	    for (i=0; i<3; i++) {           
	        if (FindPattern ((ViChar *)buffer, 0, -1, instrVersionList[i], 0, 0) >= 0) {
	            Scan (instrVersionList[i], "%s>%s", instrVer);
	            i = 9;
	        }
	    }       
	    if (i != 10) {
	        rssmp_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;      
	        return rssmp_status;
	    }       
    
	    if ((rssmp_status = rssmp_confOPT (instrSession, optsList)) < 0)              
	        return rssmp_status;

	    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
	        return rssmp_status;                
	}
	else { /* SMR connected */
		sprintf (instrVer, "SMR connected - compatibility mode active (compliant with firmware 1.06).");
		sprintf (optsList, "SMR connected - simulating all possible SMP options.");
	}

    return rssmp_status;
}

/*=========================================================================*/
/* Function: Reference Oscillator                                          */
/* Purpose:  This function selects and configures source of the reference  */
/*           frequency.                                                    */
/*=========================================================================*/
ViStatus _VI_FUNC rssmp_confRefOscillator (ViSession instrSession,
    ViBoolean refSource, ViBoolean phasAdjust, ViBoolean unit, ViReal64 phase,
    ViBoolean freqAdjust, ViInt32 extFreq, ViInt32 freqCorr)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;           
    ViUInt32 retCnt = 0;                                                 
    rssmp_instrRange instrPtr;

    if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssmp_status;

    if (rssmp_invalidViBooleanRange (refSource))        
        return VI_ERROR_PARAMETER2;        
    if (rssmp_invalidViBooleanRange (phasAdjust))        
        return VI_ERROR_PARAMETER3;        
    if (rssmp_invalidViBooleanRange (unit))        
        return VI_ERROR_PARAMETER4;        
    if (unit == VI_FALSE) {
        if (rssmp_invalidViReal64Range (phase, -360.0/180*PI, 360.0/180*PI))
            return VI_ERROR_PARAMETER5;
    }           
    else 
        if (rssmp_invalidViReal64Range (phase, -360.0, 360.0))
            return VI_ERROR_PARAMETER5;
    if (rssmp_invalidViBooleanRange (freqAdjust))        
        return VI_ERROR_PARAMETER6;        
    if (rssmp_invalidViInt32Range (extFreq, 1, 16))        
        return VI_ERROR_PARAMETER5;        
    if (rssmp_invalidViInt32Range (freqCorr, 0, 4095))        
        return VI_ERROR_PARAMETER6;        

	if ((instrPtr -> instr) == SMP) { /* SMP connected ? */

	    Fmt (buffer, "%s<:SOUR:ROSC:SOUR %s;EXT:FREQ %d[b4]MHZ;:SOUR:ROSC:INT:ADJ:STAT %s;VAL %d[b4];:SOUR:PHAS:ADJ %f%s",
	        sourceArr[refSource], extFreq, stateArr[freqAdjust], freqCorr, phase, 
	        phaseUnitArr[unit]);
	    bytesFmt = NumFmtdBytes (); 
    
	    if (phasAdjust == VI_TRUE) {
	        Fmt (buffer, "%s[a]<;:SOUR:PHAS:REF");
	        bytesFmt += NumFmtdBytes ();                            
	    }       
                
	    if ((rssmp_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
	        return rssmp_status;
	}
	else { /* SMR connected */
	    if ((rssmp_status = viPrintf (instrSession, ":SOUR:ROSC:SOUR %s"
	    	";:SOUR:ROSC:INT:ADJ:STAT %s;VAL %Ld", sourceArr[refSource],
	    	stateArr[freqAdjust], freqCorr)) < 0)
	        return rssmp_status;
	}
        
    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)
        return rssmp_status;                
        
    return rssmp_status;
}

/*=========================================================================*/
/* Function: Auxiliary Outputs                                             */
/* Purpose:  This function configures instrument's auxiliary outputs.      */
/*=========================================================================*/
ViStatus _VI_FUNC rssmp_confAuxOut (ViSession instrSession, ViBoolean blankPol,
    ViBoolean blankTime, ViBoolean VGHzScale, ViReal64 zBlank, ViReal64 zMarker)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViInt16 rssmp_optPresent = VI_FALSE;
    ViInt32 bytesFmt = 0;           
    ViUInt32 retCnt = 0;                                                 
    rssmp_instrRange instrPtr;

    if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssmp_status;

    if (rssmp_invalidViBooleanRange (blankPol))        
        return VI_ERROR_PARAMETER2;        
    if (rssmp_invalidViBooleanRange (blankTime))        
        return VI_ERROR_PARAMETER3;        
    if (rssmp_invalidViBooleanRange (VGHzScale))        
        return VI_ERROR_PARAMETER4;        
    if (rssmp_invalidViReal64Range (zBlank, -10.0, 10.0))        
        return VI_ERROR_PARAMETER5;        
    if (rssmp_invalidViReal64Range (zMarker, -10.0, 10.0))        
        return VI_ERROR_PARAMETER6;        

	if ((instrPtr -> instr) == SMP) { /* SMP connected ? */

	    if ((rssmp_status = rssmp_queryInstrOption (instrSession, 0x4000, &rssmp_optPresent)) < 0)
	        return rssmp_status;     
        
	    Fmt (buffer, "%s<:OUTP:BLAN:POL %s;:SOUR:SWE:BTIM %s;:OUTP:ZAX:BLAN %f;MARK %f",
	        polarityArr[blankPol], sweBlankTmArr[blankTime], zBlank, zMarker);
	    bytesFmt = NumFmtdBytes ();                             

	    if (rssmp_optPresent == VI_TRUE) {
	        Fmt (buffer, "%s[a]<;:OUTP3:SCAL %s", VGHzScaleArr[VGHzScale]);
	        bytesFmt += NumFmtdBytes ();                            
	    }       
                
	    if ((rssmp_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)              
	        return rssmp_status;                
    }
    else { /* SMR connected */
	    if ((rssmp_status = viPrintf (instrSession, ":OUTP3:POL:PULS %s;:SOUR:SWE:BTIM %s"
	    	";:OUTP3:SCAL %s", polarityArr[blankPol], sweBlankTmArr[blankTime],
	    	VGHzScaleArr[VGHzScale])) < 0)
	        return rssmp_status;
    }
    
    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
        return rssmp_status;                
        
    return rssmp_status;
}

/*===========================================================================*/
/* Function: Beeper                                                          */
/* Purpose:  This function activates and deactivates beeper on the instrument*/
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_confBeep (ViSession instrSession, ViBoolean beeper)
{
    ViStatus rssmp_status = VI_SUCCESS;
    rssmp_instrRange instrPtr;

    if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssmp_status;

	if ((instrPtr -> instr) == SMP) { /* SMP connected ? */

	    if (rssmp_invalidViBooleanRange (beeper))        
	        return VI_ERROR_PARAMETER2;        
        
	    if ((rssmp_status = viPrintf (instrSession, ":SYST:BEEP:STAT %s", stateArr[beeper])) < 0)               
	        return rssmp_status;       

	    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
	        return rssmp_status;
	}
	else
		return rssmp_VI_WARN_NSUP_FUNCTION;

    return rssmp_status;    
}

/*=========================================================================*/
/* Function: Outputs On/Off                                                */
/* Purpose:  This function turns instrument's outputs on or off.           */
/*=========================================================================*/
ViStatus _VI_FUNC rssmp_outState (ViSession instrSession, ViBoolean RFstate, 
    ViBoolean LFstate, ViBoolean VGHzstate)
{
    ViStatus rssmp_status = VI_SUCCESS;                       
    ViInt16 rssmp_optPresent = VI_FALSE;
        
    if (rssmp_invalidViBooleanRange (RFstate))        
        return VI_ERROR_PARAMETER2;        
    if (rssmp_invalidViBooleanRange (LFstate))        
        return VI_ERROR_PARAMETER3;        
    if (rssmp_invalidViBooleanRange (VGHzstate))        
        return VI_ERROR_PARAMETER4;        
        
    if ((rssmp_status = rssmp_queryInstrOption (instrSession, 0x4000, &rssmp_optPresent)) < 0)
        return rssmp_status;     
        
    if (rssmp_optPresent == VI_TRUE) {
        if ((rssmp_status = viPrintf (instrSession, ":OUTP1 %s;:OUTP2 %s;:OUTP3 %s", 
            stateArr[RFstate], stateArr[LFstate], stateArr[VGHzstate])) < 0)              
            return rssmp_status;       
    }
    else {
        if (VGHzstate == VI_TRUE) {
            rssmp_status = rssmp_VI_WARN_NSUP_OPTION;
            return rssmp_status;
        }
        else
            if ((rssmp_status = viPrintf (instrSession, ":OUTP1 %s;:OUTP2 %s", 
                stateArr[RFstate], stateArr[LFstate])) < 0)              
                return rssmp_status;       
    }           

    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
        return rssmp_status;                
        
    return rssmp_status;                
}

/*=========================================================================*/
/* Function: Send Trigger                                                  */
/* Purpose:  This function triggers the instrument.                        */
/*=========================================================================*/
ViStatus _VI_FUNC rssmp_sendTrig (ViSession instrSession, ViInt16 trigger)
{
    ViStatus rssmp_status = VI_SUCCESS;                       

    if (rssmp_invalidViInt16Range (trigger, 0, 2))        
        return VI_ERROR_PARAMETER2;        

    if ((rssmp_status = viPrintf (instrSession, ":TRIG:%s:IMM", trigAbortArr[trigger])) < 0)              
        return rssmp_status;       

    return rssmp_status;                
}

/*=========================================================================*/
/* Function: Abort                                                         */
/* Purpose:  This function aborts an operation which is in progress.       */
/*=========================================================================*/
ViStatus _VI_FUNC rssmp_abort (ViSession instrSession, ViInt16 trigger)
{
    ViStatus rssmp_status = VI_SUCCESS;                       

    if (rssmp_invalidViInt16Range (trigger, 0, 2))        
        return VI_ERROR_PARAMETER2;        
        
    if ((rssmp_status = viPrintf (instrSession, ":ABOR:%s", trigAbortArr[trigger])) < 0)              
        return rssmp_status;       

    return rssmp_status;                
}

/*=========================================================================*/
/* Function: Operation Complete                                            */
/* Purpose:  This function waits until all previous operations will have   */
/*           been completed.                                               */
/*=========================================================================*/
ViStatus _VI_FUNC rssmp_operationComplete (ViSession instrSession, ViInt32 maxTimeout)
{
    ViStatus rssmp_status = VI_SUCCESS;                       
    ViUInt32 retCnt = 0;
    ViUInt16 interface = VI_INTF_GPIB;
    ViUInt16 response = 0;
    ViInt32 i = 0, oldTimeout = 0;
    
    if (rssmp_invalidViReal64Range (maxTimeout, 0, 600))        
        return VI_ERROR_PARAMETER2;        
        
    if ((rssmp_status = viWrite (instrSession, (ViBuf)"*WAI;*OPC?", 10, &retCnt)) < 0)
        return rssmp_status;
        
    /* Serial interface support */
    if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_INTF_TYPE,
    	&interface)) < 0)
        return rssmp_status;

    if (interface == VI_INTF_GPIB)  /* GPIB */
        {
	    while (!(response & 0x10)) {
    
	        if ((rssmp_status = viReadSTB (instrSession, &response)) < 0)
	            return rssmp_status;        
            
	        Delay(0.1);
	        i++;
        
	        if (i/10 > maxTimeout) {
	            rssmp_status = VI_ERROR_TMO;
	            return rssmp_status;
	        }
	    }
	    
	    if ((rssmp_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
	        return rssmp_status;
	}
	else {
		if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &oldTimeout) < 0))
	        return rssmp_status;
		if ((rssmp_status = viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, maxTimeout*1000) < 0))
	        return rssmp_status;
	    if ((rssmp_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
	        return rssmp_status;
		if ((rssmp_status = viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, oldTimeout) < 0))
	        return rssmp_status;
	}

    return rssmp_status;        
}

/*=========================================================================*/
/* Function: Step                                                          */
/* Purpose:  This function configures and performes step up/down of a      */
/*           specified parameter.                                          */
/*=========================================================================*/
ViStatus _VI_FUNC rssmp_step (ViSession instrSession, ViBoolean mode,
    ViBoolean direction, ViInt16 parameter, ViReal64 stepSize)
{
    ViStatus rssmp_status = VI_SUCCESS;                       
    
    if (rssmp_invalidViBooleanRange (mode))        
        return VI_ERROR_PARAMETER2;        
    if (rssmp_invalidViBooleanRange (direction))        
        return VI_ERROR_PARAMETER3;        
    if (rssmp_invalidViInt16Range (parameter, 0, 3))        
        return VI_ERROR_PARAMETER4;        
    if (parameter > 2) {    
        if (rssmp_invalidViReal64Range (stepSize, 0.1, 10))        
            return VI_ERROR_PARAMETER5;        
    }
    else
        if (rssmp_invalidViReal64Range (stepSize, 0, 1E10))        
            return VI_ERROR_PARAMETER5;        

    if (mode == VI_TRUE) {
        if ((rssmp_status = viPrintf (instrSession, "%s %f", stepConfigArr[parameter], stepSize)) < 0)
            return rssmp_status;       
    }
    else 
        if ((rssmp_status = viPrintf (instrSession, "%s %s", stepExeArr[parameter], directArr[direction])) < 0)
            return rssmp_status;       

    if ((rssmp_status = rssmp_sysStatus (instrSession)) < 0)              
        return rssmp_status;                
        
    return rssmp_status;                
}

/*===========================================================================*/
/* Function: Write To Instrument                                             */
/* Purpose:  This function writes a command string to the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_writeInstrData (ViSession instrSession, ViString writebuffer)
{
    ViStatus rssmp_status = VI_SUCCESS;
    
    if ((rssmp_status = viPrintf (instrSession, "%s", writebuffer)) < 0)
        return rssmp_status;

    return rssmp_status;
}

/*===========================================================================*/
/* Function: Read Instrument buffer                                          */
/* Purpose:  This function reads the output buffer of the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_readInstrData (ViSession instrSession, ViInt16 numBytes,
                    ViChar _VI_FAR rdBuf[], ViPInt32 bytesRead)
{
    ViStatus rssmp_status = VI_SUCCESS;
	ViUInt32	 tmpVal;
	
	if (bytesRead)
		*bytesRead = 0L;
        
    if ((rssmp_status = viRead (instrSession, (ViBuf)rdBuf, numBytes, &tmpVal)) < 0)
        return rssmp_status;
	
	if (bytesRead)
		*bytesRead = (ViInt32)tmpVal;

    return rssmp_status;
}

/*===========================================================================*/
/* Function: Reset                                                           */
/* Purpose:  This function resets the instrument.  If the reset function     */
/*           is not supported by the instrument, this function returns       */
/*           the warning VI_WARN_NSUP_RESET.                                 */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_reset (ViSession instrSession)
{
    ViUInt32 retCnt = 0;
    ViStatus rssmp_status = VI_SUCCESS;

    /*  Initialize the instrument to a known state.  */
    if ((rssmp_status = viWrite (instrSession, (ViBuf)"*RST", 4, &retCnt)) < 0)
        return rssmp_status;

    if ((rssmp_status = rssmp_defaultInstrSetup (instrSession)) < 0)  
        return rssmp_status;
        
    return rssmp_status;
}

/*===========================================================================*/
/* Function: Self-Test                                                       */
/* Purpose:  This function executes the instrument self-test and returns     */
/*           the result.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_self_test (ViSession instrSession, ViPInt16 result,
                                   ViChar _VI_FAR message[])
{
    ViUInt32 retCnt = 0;
    ViStatus rssmp_status = VI_SUCCESS;
	ViInt16	 tmpVal;

    *result = 0;

    if ((rssmp_status = viWrite (instrSession, (ViBuf)":TEST:RAM?", 10, &retCnt)) < 0)
        return rssmp_status;
    if ((rssmp_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rssmp_status;

	if (Scan (buffer, "%s>%i[b2]", &tmpVal) < 0)
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	if (tmpVal != 0)
		{
		*result = tmpVal;
		strcpy(message, "RAM test error.");
		return rssmp_status;
		}

    if ((rssmp_status = viWrite (instrSession, (ViBuf)":TEST:ROM?", 10, &retCnt)) < 0)
        return rssmp_status;
    if ((rssmp_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rssmp_status;
    
	if (Scan (buffer, "%s>%i[b2]", &tmpVal) < 0)
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if (tmpVal != 0)
		{
		*result = tmpVal;
		strcpy(message, "ROM test error.");
		return rssmp_status;
		}

    if ((rssmp_status = viWrite (instrSession, (ViBuf)":TEST:BATT?", 11, &retCnt)) < 0)
        return rssmp_status;
    if ((rssmp_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rssmp_status;
    
    if (Scan (buffer, "%s>%i[b2]", &tmpVal) < 0)
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if (tmpVal != 0)
		{
		*result = tmpVal;
		strcpy(message, "Battery test error.");
		return rssmp_status;
		}
	
	strcpy(message, "No error.");
    
    return rssmp_status;
}

/*===========================================================================*/
/* Function: Error Query                                                     */
/* Purpose:  This function queries the instrument error queue, and returns   */
/*           the result. If the error query function is not supported by the  */
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_ERROR_QUERY.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_error_query (ViSession instrSession, ViPInt32 errCode,
                    ViChar _VI_FAR errMessage[])
{
    ViUInt32 retCnt = 0;
    ViStatus rssmp_status = VI_SUCCESS;

    if ((rssmp_status = viWrite (instrSession, (ViBuf)":SYST:ERR?", 10, &retCnt)) < 0)
        return rssmp_status;

    if ((rssmp_status = viScanf (instrSession, "%ld,\"%[^\"]", errCode, errMessage)) < 0)
        return rssmp_status;

    return rssmp_status;
}

/*===========================================================================*/
/* Function: Error Message                                                   */
/* Purpose:  This function translates the error return value from the        */
/*           instrument driver into a user-readable string.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_error_message (ViSession instrSession, ViStatus errorCode,
                    ViChar _VI_FAR errMessage[])
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViInt16 i;
    static rssmp_tStringValPair statusDescArray[] = {
        {VI_WARN_NSUP_ID_QUERY,     "WARNING: ID Query not supported"},
        {VI_WARN_NSUP_RESET,        "WARNING: Reset not supported"},
        {VI_WARN_NSUP_SELF_TEST,    "WARNING: Self-test not supported"},
        {VI_WARN_NSUP_ERROR_QUERY,  "WARNING: Error Query not supported"},     
        {VI_WARN_NSUP_REV_QUERY,    "WARNING: Revision Query not supported"},
        {VI_ERROR_PARAMETER1,   "ERROR: Parameter 1 out of range"},
        {VI_ERROR_PARAMETER2,   "ERROR: Parameter 2 out of range"},
        {VI_ERROR_PARAMETER3,   "ERROR: Parameter 3 out of range"},
        {VI_ERROR_PARAMETER4,   "ERROR: Parameter 4 out of range"},
        {VI_ERROR_PARAMETER5,   "ERROR: Parameter 5 out of range"},
        {VI_ERROR_PARAMETER6,   "ERROR: Parameter 6 out of range"},
        {VI_ERROR_PARAMETER7,   "ERROR: Parameter 7 out of range"},
        {VI_ERROR_PARAMETER8,   "ERROR: Parameter 8 out of range"},
        {VI_ERROR_INSTR_PARAMETER9,   "ERROR: Parameter 9 out of range"},
        {VI_ERROR_INSTR_PARAMETER10,  "ERROR: Parameter 10 out of range"},
        {VI_ERROR_INSTR_PARAMETER11,  "ERROR: Parameter 11 out of range"},
        {VI_ERROR_INSTR_PARAMETER12,  "ERROR: Parameter 12 out of range"},
        {VI_ERROR_INSTR_PARAMETER13,  "ERROR: Parameter 13 out of range"},
        {VI_ERROR_INSTR_PARAMETER14,  "ERROR: Parameter 14 out of range"},
        {VI_ERROR_INSTR_PARAMETER15,  "ERROR: Parameter 15 out of range"},
        {VI_ERROR_FAIL_ID_QUERY,"ERROR: Identification query failed"},
        {VI_ERROR_INV_RESPONSE, "ERROR: Interpreting instrument response"},
        {VI_ERROR_INSTR_FILE_OPEN,    "ERROR: Opening the specified file"},
        {VI_ERROR_INSTR_FILE_WRITE,   "ERROR: Writing to the specified file"},
        {VI_ERROR_INSTR_INTERPRETING_RESPONSE, "ERROR: Interpreting the instrument's response"},
        {rssmp_VI_WARN_NSUP_OPTION,   "WARNING: Not supported instrument option"},
        {rssmp_VI_WARN_NSUP_FUNCTION,   "WARNING: Currently not supported function"},
        {rssmp_ERROR_INSTRUMENT_ERROR,  "ERROR: Instrument specific error"},
        {VI_NULL, VI_NULL}
    };

    rssmp_status = viStatusDesc (instrSession, errorCode, errMessage);
    if (rssmp_status == VI_WARN_UNKNOWN_STATUS) {
        for (i=0; statusDescArray[i].stringName; i++) {
            if (statusDescArray[i].stringVal == errorCode) {
                Fmt (errMessage, "%s<%s", statusDescArray[i].stringName);
                return (VI_SUCCESS);
            }
        }
        Fmt (errMessage, "%s<Unknown Error 0x%x[uw8p0]", errorCode);
        return (VI_WARN_UNKNOWN_STATUS);
    }
    
    rssmp_status = VI_SUCCESS;
    return rssmp_status;
}

/*===========================================================================*/
/* Function: Revision Query                                                  */
/* Purpose:  This function returns the driver and instrument revisions.      */
/*           If the revision query function is not supported by the          */ 
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_REV_QUERY.                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_revision_query (ViSession instrSession,
    ViChar _VI_FAR driverRev[], ViChar _VI_FAR instrRev[])
{
    ViUInt32 retCnt = 0;
    ViStatus rssmp_status = VI_SUCCESS;

    if ((rssmp_status = viWrite (instrSession, (ViBuf)"*IDN?", 5, &retCnt)) < 0)
        return rssmp_status;

    if ((rssmp_status = viScanf (instrSession, "%*[^,],%*[^,],%*[^,],%[^\n]", instrRev)) < 0)
        return rssmp_status;

    Fmt (driverRev, "%s<%s", rssmp_REVISION);
    
    return rssmp_status;
}

/*===========================================================================*/
/* Function: Close                                                           */
/* Purpose:  This function closes the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rssmp_close (ViSession instrSession)
{
    rssmp_instrRange instrPtr;
    ViSession rmSession;
    ViStatus rssmp_status = VI_SUCCESS;

    if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_RM_SESSION, &rmSession)) < 0)
        return rssmp_status;
    if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssmp_status;
    
    if (instrPtr != NULL)    
        free (instrPtr);
    
    rssmp_status = viClose (instrSession);
    viClose (rmSession);

    return rssmp_status;
}

/*****************************************************************************/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE   */
/*           or VI_FALSE. If the value is out of range, the return value is    */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                    */
/*===========================================================================*/
ViBoolean rssmp_invalidViBooleanRange (ViBoolean val)
{
    return (ViBoolean)((val != VI_FALSE && val != VI_TRUE) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Signed Integer Value Out Of Range - ViInt16               */
/* Purpose:  This function checks a short signed integer value to see if it  */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return       */
/*           value is VI_FALSE.                                                */
/*===========================================================================*/
ViBoolean rssmp_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range,  the return value is VI_TRUE, otherwise the return      */
/*           value is VI_FALSE.                                                */
/*===========================================================================*/
ViBoolean rssmp_invalidViInt32Range  (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Unsigned Integer Value Out Of Range - ViUInt16            */
/* Purpose:  This function checks a short unsigned integer value to see if it*/  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range,  the return value is VI_TRUE, otherwise the return      */
/*           value is VI_FALSE.                                                */
/*===========================================================================*/
ViBoolean rssmp_invalidViUInt16Range  (ViUInt16 val, ViUInt16 min, ViUInt16 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Unsigned Integer Value Out Of Range - ViUInt32             */
/* Purpose:  This function checks a long unsigned integer value to see if it */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range,  the return value is VI_TRUE, otherwise the return      */
/*           value is VI_FALSE.                                                */
/*===========================================================================*/
ViBoolean rssmp_invalidViUInt32Range  (ViUInt32 val, ViUInt32 min, ViUInt32 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Float) Value Out Of Range - ViReal32                      */
/* Purpose:  This function checks a real (float) value to see if it lies     */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value    */
/*           is VI_FALSE.                                                      */
/*===========================================================================*/
ViBoolean rssmp_invalidViReal32Range  (ViReal32 val, ViReal32 min, ViReal32 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value    */
/*           is VI_FALSE.                                                      */
/*===========================================================================*/
ViBoolean rssmp_invalidViReal64Range  (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Initialize Clean Up                                             */
/* Purpose:  This function is used only by the rssmp_init function.  When    */
/*           an error is detected this function is called to close the       */
/*           open resource manager and instrument object sessions and to     */
/*           set the instrSession that is returned from rssmp_init to        */
/*           VI_NULL.                                                        */
/*===========================================================================*/
ViStatus rssmp_initCleanUp (ViSession openRMSession,
                    ViPSession openInstrSession, ViStatus currentStatus)
{
    rssmp_instrRange instrPtr;
    
    if (viGetAttribute (*openInstrSession, VI_ATTR_USER_DATA, &instrPtr) >= 0)
        if (instrPtr != NULL) 
            free (instrPtr);
   
    viClose (*openInstrSession);
    viClose (openRMSession);
    *openInstrSession = VI_NULL;
    
    return currentStatus;
}

/*****************************************************************************/
/*----------- INSTRUMENT-DEPENDENT UTILITY ROUTINES -------------------------*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Default Instrument Setup                                        */
/* Purpose:  This function sends a default setup to the instrument.  This    */
/*           function is called by the rssmp_reset operation and by the      */
/*           rssmp_init function if the reset option has not been            */
/*           selected.  This function is useful for configuring any          */
/*           instrument settings that are required by the rest of the        */
/*           instrument driver functions such as turning headers ON or OFF   */
/*           or using the long or short form for commands, queries, and data.*/                                    
/*===========================================================================*/
ViStatus rssmp_defaultInstrSetup (ViSession instrSession)
{
    ViStatus rssmp_status = VI_SUCCESS;
    ViInt16 rssmp_optPresent = VI_FALSE;
    ViUInt32 retCnt = 0;
    ViInt32 i = 0;
    ViChar opt[BUFFER_SIZE];
    rssmp_instrRange instrPtr;

    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */
    if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssmp_status;
    
    if (instrPtr == NULL) 
        instrPtr = malloc (sizeof (struct rssmp_statusDataRanges));
    instrPtr -> option = 0;
    Fmt (instrPtr -> instrDriverRevision, "%s<%s", rssmp_REVISION);
    
    if (viSetAttribute (instrSession, VI_ATTR_USER_DATA, (ViUInt32)instrPtr) != 0)
        return rssmp_status;                                       
    
    /*----------- Clear Status Structures ---------------------------------*/

    if ((rssmp_status = viWrite (instrSession, (ViBuf)"*CLS", 4, &retCnt)) < 0)
        return rssmp_status;
    if ((rssmp_status = viWrite (instrSession, (ViBuf)"*IDN?", 5, &retCnt)) < 0)
        return rssmp_status;
    if ((rssmp_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rssmp_status;

	/* If SMR not recognized, default instrument will be SMP. */
	(instrPtr -> instr) = (!strstr (buffer, "SMR")) ? SMP : SMR;

    if (Scan (buffer, "%s>%s", opt) < 0) {
        rssmp_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return rssmp_status;
    }        

	if ((instrPtr -> instr) == SMR) { /* Bypassing known firmware bugs. */
	    if ((rssmp_status = viQueryf (instrSession, "*IDN?", "%*[^,],%*[^,],%*[^,],%[^\n]", buffer)) < 0)
        	return rssmp_status;
    
	    /* VI_FALSE ... SMR Firmware version 1.07 */
	    /* VI_TRUE ...  SMR Firmware version 1.08 or higher */
    
	    instrPtr -> firmware = (!strstr (buffer, "1.07")) ? VI_TRUE : VI_FALSE ;
	}
	else
		instrPtr -> firmware = VI_TRUE; /* Compliant to SMP. */

	if ((instrPtr -> instr) == SMP) {
	    for (i=0; i<3; i++) {
	        if (FindPattern (opt, 0, -1, instrVersionList[i], 0, 0) >= 0) {
	            instrPtr -> maxFrequency = maxFrequencyList[i];
	            instrPtr -> maxLevel = maxLevelList[i];
	        }
	    }
	}
	else { /* If SMR use max. possible ranges. */
        instrPtr -> minFrequency = minFrequencyList[1]; /* 10 MHz */
        instrPtr -> maxFrequency = maxFrequencyList[2]; /* 40 GHz */
        instrPtr -> minLevel = minLevelList[1]; /* -130 dBm */
        instrPtr -> maxLevel = maxLevelList[0]; /* 27 dBm */
	}

	/* If SMR simulate all options included */
	    /*----------- Option Identification Query -----------------------------*/

	    if ((rssmp_status = rssmp_confOPT (instrSession, opt)) < 0)
	        return rssmp_status;
    
	    for (i=0; i<rssmp_NUMB_OF_AVAILABLE_OPTIONS; i++) 
	        if (FindPattern (opt, 0, -1, optionsList[i], 0, 0) >= 0)
	            instrPtr -> option = (ViInt16)(instrPtr -> option + (ViInt16)pow(2,i));

	if ((instrPtr -> instr) == SMP) { /* Valid only for SMP */
	    /*----------- Option SMP-B11 Identification Query ---------------------*/
    
	    if ((rssmp_status = rssmp_queryInstrOption (instrSession, 0x80, &rssmp_optPresent)) < 0)
	        return rssmp_status;     

	    if (rssmp_optPresent == VI_TRUE)
	        instrPtr -> minFrequency = minFrequencyList[1];
	    else
	        instrPtr -> minFrequency = minFrequencyList[0];
        
	    /*----------- Option SMP-B15 Identification Query ---------------------*/
    
	    if ((rssmp_status = rssmp_queryInstrOption (instrSession, 0x800, &rssmp_optPresent)) < 0)
	        return rssmp_status;     

	    if (rssmp_optPresent == VI_TRUE)
	        instrPtr -> minLevel = minLevelList[1];
	    else
	        instrPtr -> minLevel = minLevelList[0];
	}

    return rssmp_status;
}

/*=========================================================================*/
/* Function: Status                                                        */
/* Purpose:  This function reads the Standard Event Register and checks    */
/*           for error.                                                    */
/*=========================================================================*/
ViStatus rssmp_sysStatus (ViSession instrSession)
{
    ViStatus rssmp_status = VI_SUCCESS;     
    ViUInt32 retCnt = 0;
        
    ViInt32 esr;

    /*--------------- Read the Standard Event Register --------------------*/

    if ((rssmp_status = viWrite (instrSession, (ViBuf)"*ESR?", 5, &retCnt)) < 0)
        return rssmp_status;
    if ((rssmp_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rssmp_status;
        
    if (Scan (buffer, "%s>%d[b4]", &esr) != 1) {
        rssmp_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return rssmp_status;
    }        

    /*---------------- Check if any error bit was asserted ----------------*/

    if ((esr & 0x3C) != 0)
        rssmp_status = rssmp_ERROR_INSTRUMENT_ERROR;
    else
       rssmp_status = VI_SUCCESS;

    return rssmp_status;
}

/*=========================================================================*/
/* Function: Query Instrument Option                                       */
/* Purpose:  This function checks if an option is included in the          */
/*           instrument.                                                   */
/*=========================================================================*/
ViStatus rssmp_queryInstrOption (ViSession instrSession, ViInt16 queryBit, ViPInt16 bitValue)
{
    ViStatus rssmp_status = VI_SUCCESS;                       
    rssmp_instrRange instrPtr;
    
    if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssmp_status;
    
    if ((instrPtr -> option & queryBit) != 0)
        *bitValue = VI_TRUE;
    else
        *bitValue = VI_FALSE;
    
    return rssmp_status;        
}

/*=========================================================================*/
/* Function: Query Instrument Version (Frequency)                          */
/* Purpose:  This function queries the instrument version.                 */
/*=========================================================================*/
ViStatus rssmp_queryInstrVersionFreq (ViSession instrSession, ViPReal64 minFreq, ViPReal64 maxFreq)
{
    ViStatus rssmp_status = VI_SUCCESS;                       
    rssmp_instrRange instrPtr;
    
    if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssmp_status;
    
    *minFreq = instrPtr -> minFrequency;    
    *maxFreq = instrPtr -> maxFrequency;
    
    return rssmp_status;        
}

/*=========================================================================*/
/* Function: Query Instrument Version (Level)                              */
/* Purpose:  This function queries the instrument version.                 */
/*=========================================================================*/
ViStatus rssmp_queryInstrVersionLev (ViSession instrSession, ViPReal64 minLev, ViPReal64 maxLev)
{
    ViStatus rssmp_status = VI_SUCCESS;                       
    rssmp_instrRange instrPtr;
    
    if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssmp_status;
    
    *minLev = instrPtr -> minLevel; 
    *maxLev = instrPtr -> maxLevel;
    
    return rssmp_status;        
}

/*===========================================================================*/
/* Function: Instrument Options                                              */
/* Purpose:  This function returns the list of options which the instrument  */
/*           is equipped with.                                               */
/*===========================================================================*/
ViStatus rssmp_confOPT (ViSession instrSession, ViString optsList)
{
    ViStatus rssmp_status = VI_SUCCESS;                            
    ViUInt32 retCnt = 0;
    ViBoolean i = VI_TRUE;
    ViInt32 searchIndex = 0;
    ViInt32 storeIndex = 0;
    rssmp_instrRange instrPtr;

    if ((rssmp_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssmp_status;
    
    FillBytes ((ViChar *)buffer, 0, BUFFER_SIZE, '\0');
    
	if ((instrPtr -> instr) == SMP) {
	    if ((rssmp_status = viWrite (instrSession, (ViBuf)"*OPT?", 5, &retCnt)) < 0)
	        return rssmp_status;
	    if ((rssmp_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
	        return rssmp_status;
            
	    if (Fmt (optsList, "%s<%s", buffer) < 0) {
	        rssmp_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;      
	        return rssmp_status;
	    }
	} else 	/* If SMR simulate all options included. */
		Fmt (optsList, "SM-B1,SM-B2,SM-B2,SM-B5,SM-BX,SM-BX,SM-BX,SMP-B11,SMP-B12,SMP-B13,SMP-B14,SMP-B15,SMP-BX,SMP-B17,SMP-B18");

    while (i) {
        
        searchIndex = FindPattern (optsList, searchIndex, -1, "S", 0, 0);   
    
        if (searchIndex < 0) {
            FillBytes (optsList, storeIndex - 1, 1, 0);
            break;
        }       
    
        CopyString (optsList, storeIndex, optsList, searchIndex, -1);
    
        storeIndex = FindPattern (optsList, storeIndex, -1, ",", 0, 0);
    
        if (storeIndex < 0) {
            storeIndex = StringLength (optsList);
            FillBytes (optsList, storeIndex - 1, 1, 0);
            break;
        }
        
        storeIndex++;
    
        searchIndex = storeIndex;
    }

    return rssmp_status;
}
    
/*****************************************************************************/
/*=== END INSTRUMENT DRIVER SOURCE CODE =====================================*/
/*****************************************************************************/
